@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2019 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,56 +23,53 @@
 
 #include "swarm.h"
 
-//#define HASH hash_fnv_1a_64
-#define HASH hash_cityhash64
+constexpr unsigned int memchunk {1 << 20};  // 1 megabyte
+constexpr unsigned int linealloc {2048};
+constexpr long unsigned int n_chars {INT8_MAX + 1};  // 128 ascii chars
 
-#define MEMCHUNK 1048576
-#define LINEALLOC LINE_MAX
+auto make_nt_map () -> std::array<signed char, n_chars> {
+    // set the 128 ascii chars to '-1' except Aa, Cc, Gg, Tt and Uu
+    std::array<signed char, n_chars> ascii_map;
+    ascii_map.fill(-1);
+    ascii_map['A'] = 1;
+    ascii_map['a'] = 1;
+    ascii_map['C'] = 2;
+    ascii_map['c'] = 2;
+    ascii_map['G'] = 3;
+    ascii_map['g'] = 3;
+    ascii_map['T'] = 4;
+    ascii_map['t'] = 4;
+    ascii_map['U'] = 4;
+    ascii_map['u'] = 4;
+    return ascii_map;
+    }
 
-static signed char map_nt[256] =
-  {
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1,  1, -1,  2, -1, -1, -1,  3, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1,  4,  4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1,  1, -1,  2, -1, -1, -1,  3, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1,  4,  4, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1,
-    -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1, -1
-  };
+const auto map_nt = make_nt_map();
 
-static unsigned int sequences = 0;
-static uint64_t nucleotides = 0;
-static uint64_t headerchars = 0;
-static unsigned int longest = 0;
-static uint64_t longestheader = 0;
-static char * datap = nullptr;
-static int missingabundance = 0;
-static uint64_t missingabundance_lineno = 0;
-static char * missingabundance_header = nullptr;
+static unsigned int sequences {0};
+static uint64_t nucleotides {0};
+static uint64_t headerchars {0};
+static unsigned int longest {0};
+static unsigned int longestheader {0};
+static char * datap {nullptr};
+static int missingabundance {0};
+static uint64_t missingabundance_lineno {0};
+static char * missingabundance_header {nullptr};
 
-seqinfo_t * seqindex = nullptr;
-qgramvector_t * qgrams = nullptr;
+seqinfo_t * seqindex {nullptr};
+qgramvector_t * qgrams {nullptr};
 
-int db_compare_abundance(const void * a, const void * b);
+auto db_compare_abundance(const void * a, const void * b) -> int;
 
-bool find_swarm_abundance(const char * header,
+auto find_swarm_abundance(const char * header,
                           int * start,
                           int * end,
-                          int64_t * number);
+                          int64_t * number) -> bool;
 
-bool find_usearch_abundance(const char * header,
+auto find_usearch_abundance(const char * header,
                             int * start,
                             int * end,
-                            int64_t * number);
+                            int64_t * number) -> bool;
 void find_abundance(struct seqinfo_s * sp, uint64_t lineno);
 
 void fprint_id(FILE * stream, uint64_t x)
@@ -81,13 +78,17 @@ void fprint_id(FILE * stream, uint64_t x)
   char * h = sp->header;
   int hdrlen = sp->headerlen;
 
-  if (opt_append_abundance && (sp->abundance_start == sp->abundance_end))
-    if (opt_usearch_abundance)
+  if (opt_append_abundance != 0 && (sp->abundance_start == sp->abundance_end)) {
+    if (opt_usearch_abundance) {
       fprintf(stream, "%.*s;size=%" PRIu64 ";", hdrlen, h, sp->abundance);
-    else
+    }
+    else {
       fprintf(stream, "%.*s_%" PRIu64, hdrlen, h, sp->abundance);
-  else
+    }
+  }
+  else {
     fprintf(stream, "%.*s", hdrlen, h);
+  }
 }
 
 void fprint_id_noabundance(FILE * stream, uint64_t x)
@@ -104,15 +105,17 @@ void fprint_id_noabundance(FILE * stream, uint64_t x)
       if (opt_usearch_abundance)
         {
           /* print semicolon if the abundance is not at either end */
-          if ((sp->abundance_start > 0) && (sp->abundance_end < hdrlen))
+          if ((sp->abundance_start > 0) && (sp->abundance_end < hdrlen)) {
             fprintf(stream, ";");
+          }
 
           /* print remaining part */
           fprintf(stream, "%.*s", hdrlen - sp->abundance_end, h + sp->abundance_end);
         }
     }
-  else
+  else {
     fprintf(stream, "%.*s", hdrlen, h);
+  }
 }
 
 void fprint_id_with_new_abundance(FILE * stream,
@@ -121,7 +124,7 @@ void fprint_id_with_new_abundance(FILE * stream,
 {
   seqinfo_t * sp = seqindex + seqno;
 
-  if (opt_usearch_abundance)
+  if (opt_usearch_abundance) {
     fprintf(stream,
             "%.*s%ssize=%" PRIu64 ";%.*s",
             sp->abundance_start,
@@ -130,31 +133,38 @@ void fprint_id_with_new_abundance(FILE * stream,
             abundance,
             sp->headerlen - sp->abundance_end,
             sp->header + sp->abundance_end);
-  else
+  }
+  else {
     fprintf(stream,
             "%.*s_%" PRIu64,
             sp->abundance_start,
             sp->header,
             abundance);
+  }
 }
 
-int db_compare_abundance(const void * a, const void * b)
+auto db_compare_abundance(const void * a, const void * b) -> int
 {
-  const seqinfo_t * x = reinterpret_cast<const seqinfo_t *>(a);
-  const seqinfo_t * y = reinterpret_cast<const seqinfo_t *>(b);
+  const auto * x = reinterpret_cast<const seqinfo_t *>(a);
+  const auto * y = reinterpret_cast<const seqinfo_t *>(b);
+  int status {0};
 
-  if (x->abundance > y->abundance)
-    return -1;
-  else if (x->abundance < y->abundance)
-    return +1;
-  else
-    return strcmp(x->header, y->header);
+  if (x->abundance > y->abundance) {
+    status = -1;
+  }
+  else if (x->abundance < y->abundance) {
+    status = +1;
+  }
+  else {
+    status = strcmp(x->header, y->header);
+  }
+  return status;
 }
 
-bool find_swarm_abundance(const char * header,
+auto find_swarm_abundance(const char * header,
                           int * start,
                           int * end,
-                          int64_t * number)
+                          int64_t * number) -> bool
 {
   /*
     Identify the first occurence of the pattern (_)([0-9]+)$
@@ -165,26 +175,28 @@ bool find_swarm_abundance(const char * header,
   * end = 0;
   * number = 0;
 
-  const char * digit_chars = "0123456789";
+  constexpr unsigned int max_digits {20};  // 20 digits at most (abundance > 10^20)
+  const std::string digit_chars = "0123456789";
 
-  if (!header)
-    return false;
-
-  if (strlen(header) >= INT_MAX)
+  if (header == nullptr) {
     return false;
+  }
 
   const char * us = strrchr(header, '_');
 
-  if (! us)
+  if (us == nullptr) {
     return false;
+  }
 
-  size_t digits = strspn(us + 1, digit_chars);
+  size_t digits = strspn(us + 1, digit_chars.c_str());
 
-  if (digits > 20)
+  if (digits > max_digits) {
     return false;
+  }
 
-  if (us[digits + 1])
+  if (us[digits + 1] != 0) {
     return false;
+  }
 
   int64_t s = us - header;
   int64_t e = s + 1 + static_cast<int64_t>(digits);
@@ -196,33 +208,35 @@ bool find_swarm_abundance(const char * header,
   return true;
 }
 
-bool find_usearch_abundance(const char * header,
+auto find_usearch_abundance(const char * header,
                             int * start,
                             int * end,
-                            int64_t * number)
+                            int64_t * number) -> bool
 {
   /*
     Identify the first occurence of the pattern (^|;)size=([0-9]+)(;|$)
     in the header string.
   */
 
-  if (! header)
+  if (header == nullptr) {
     return false;
+  }
 
-  const char * attribute = "size=";
-  const char * digit_chars = "0123456789";
+  const std::string attribute {"size="};
+  const std::string digit_chars = "0123456789";
 
   uint64_t hlen = strlen(header);
-  uint64_t alen = strlen(attribute);
+  uint64_t alen = attribute.length();
   uint64_t i = 0;
 
   while (i + alen < hlen)
     {
-      const char * r = strstr(header + i, attribute);
+      const char * r = strstr(header + i, attribute.c_str());
 
       /* no match */
-      if (r == nullptr)
+      if (r == nullptr) {
         break;
+      }
 
       i = static_cast<uint64_t>(r - header);
 
@@ -233,7 +247,7 @@ bool find_usearch_abundance(const char * header,
           continue;
         }
 
-      uint64_t digits = strspn(header + i + alen, digit_chars);
+      uint64_t digits = strspn(header + i + alen, digit_chars.c_str());
 
       /* check for at least one digit */
       if (digits == 0)
@@ -250,11 +264,13 @@ bool find_usearch_abundance(const char * header,
         }
 
       /* ok */
-      if (i > 0)
+      if (i > 0) {
         * start = static_cast<int>(i - 1);
-      else
+      }
+      else {
         * start = 0;
-      * end   = static_cast<int>(MIN(i + alen + digits + 1, hlen));
+      }
+      * end   = static_cast<int>(std::min(i + alen + digits + 1, hlen));
       * number = atol(header + i + alen);
       return true;
     }
@@ -278,8 +294,9 @@ void find_abundance(struct seqinfo_s * sp, uint64_t lineno)
 
       if (find_usearch_abundance(header, & start, & end, & number))
         {
-          if (number > 0)
+          if (number > 0) {
             abundance = number;
+          }
           else
             {
               fprintf(stderr,
@@ -297,8 +314,9 @@ void find_abundance(struct seqinfo_s * sp, uint64_t lineno)
 
       if (find_swarm_abundance(header, & start, & end, & number))
         {
-          if (number > 0)
+          if (number > 0) {
             abundance = number;
+          }
           else
             {
               fprintf(stderr,
@@ -316,8 +334,9 @@ void find_abundance(struct seqinfo_s * sp, uint64_t lineno)
       start = sp->headerlen;
       end = start;
 
-      if (opt_append_abundance)
+      if (opt_append_abundance != 0) {
         abundance = opt_append_abundance;
+      }
       else
         {
           missingabundance++;
@@ -334,13 +353,13 @@ void find_abundance(struct seqinfo_s * sp, uint64_t lineno)
   sp->abundance_end = end;
 }
 
-void db_read(const char * filename)
+void db_read(const char * filename, struct Parameters const & p)
 {
   /* allocate space */
 
-  uint64_t dataalloc = MEMCHUNK;
+  uint64_t dataalloc {memchunk};
   datap = static_cast<char *>(xmalloc(dataalloc));
-  uint64_t datalen = 0;
+  uint64_t datalen {0};
 
   longest = 0;
   longestheader = 0;
@@ -348,24 +367,25 @@ void db_read(const char * filename)
   nucleotides = 0;
   headerchars = 0;
 
-  FILE * fp = nullptr;
-  if (filename)
+  FILE * fp {nullptr};
+  if (filename != nullptr)
     {
       fp = fopen_input(filename);
-      if (!fp)
+      if (fp == nullptr)
         {
           fprintf(stderr, "\nError: Unable to open input data file (%s).\n", filename);
           exit(1);
         }
     }
-  else
+  else {
     fp = stdin;
+  }
 
   /* get file size */
 
   struct stat fs;
 
-  if (fstat(fileno(fp), & fs))
+  if (fstat(fileno(fp), & fs) != 0)
     {
       fprintf(stderr, "\nUnable to fstat on input file (%s)\n", filename);
       exit(1);
@@ -373,39 +393,48 @@ void db_read(const char * filename)
   bool is_regular = S_ISREG(fs.st_mode);
   int64_t filesize = is_regular ? fs.st_size : 0;
 
-  if (! is_regular)
+  if (! is_regular) {
     fprintf(logfile, "Waiting for data... (Hit Ctrl-C and run swarm -h if you meant to read data from a file.)\n");
+  }
 
-  char line[LINEALLOC];
+  char line[linealloc];
   line[0] = 0;
-  if (!fgets(line, LINEALLOC, fp))
+  if (fgets(line, linealloc, fp) == nullptr) {
     line[0] = 0;
+  }
 
-  unsigned int lineno = 1;
+  unsigned int lineno {1};
 
   progress_init("Reading sequences:", static_cast<uint64_t>(filesize));
 
-  while(line[0])
+  while(line[0] != 0)
     {
       /* read header */
       /* the header ends at a space, cr, lf or null character */
 
-      if (line[0] != '>')
+      if (line[0] != '>') {
         fatal("Illegal header line in fasta file.");
+      }
 
-      uint64_t headerlen = strcspn(line + 1, " \r\n");
+      auto headerlen = static_cast<unsigned int>
+        (strcspn(line + 1, " \r\n"));
+
+      if (headerlen >= linealloc - 2) {
+        fatal("The FASTA header line is too long.");
+      }
 
       headerchars += headerlen;
 
-      if (headerlen > longestheader)
+      if (headerlen > longestheader) {
         longestheader = headerlen;
+      }
 
 
       /* store the line number */
 
       while (datalen + sizeof(unsigned int) > dataalloc)
         {
-          dataalloc += MEMCHUNK;
+          dataalloc += memchunk;
           datap = static_cast<char *>(xrealloc(datap, dataalloc));
         }
       memcpy(datap + datalen, & lineno, sizeof(unsigned int));
@@ -416,7 +445,7 @@ void db_read(const char * filename)
 
       while (datalen + headerlen + 1 > dataalloc)
         {
-          dataalloc += MEMCHUNK;
+          dataalloc += memchunk;
           datap = static_cast<char *>(xrealloc(datap, dataalloc));
         }
       memcpy(datap + datalen, line + 1, headerlen);
@@ -427,18 +456,19 @@ void db_read(const char * filename)
       /* get next line */
 
       line[0] = 0;
-      if (!fgets(line, LINEALLOC, fp))
+      if (fgets(line, linealloc, fp) == nullptr) {
         line[0] = 0;
+      }
       lineno++;
 
 
       /* store a dummy sequence length */
 
-      unsigned int length = 0;
+      unsigned int length {0};
 
       while (datalen + sizeof(unsigned int) > dataalloc)
         {
-          dataalloc += MEMCHUNK;
+          dataalloc += memchunk;
           datap = static_cast<char *>(xrealloc(datap, dataalloc));
         }
       uint64_t datalen_seqlen = datalen;
@@ -448,17 +478,21 @@ void db_read(const char * filename)
 
       /* read and store sequence */
 
-      uint64_t nt_buffer = 0;
-      unsigned int nt_bufferlen = 0;
-      const unsigned int nt_buffersize = 4 * sizeof(nt_buffer);
+      uint64_t nt_buffer {0};
+      unsigned int nt_bufferlen {0};
+      const unsigned int nt_buffersize {4 * sizeof(nt_buffer)};
+      constexpr int new_line {10};
+      constexpr int carriage_return {13};
+      constexpr int start_chars_range {32};  // visible ascii chars: 32-126
+      constexpr int end_chars_range {126};
 
-      while (line[0] && (line[0] != '>'))
+      while ((line[0] != 0) && (line[0] != '>'))
         {
-          unsigned char c;
-          char * p = line;
-          while((c = static_cast<unsigned char>(*p++)))
-	    {
-              signed char m;
+          unsigned char c {0};
+          char * pl = line;
+          while((c = static_cast<unsigned char>(*pl++)) != 0U)
+            {
+              signed char m {0};
               if ((m = map_nt[static_cast<unsigned int>(c)]) >= 0)
                 {
                   nt_buffer |= ((static_cast<uint64_t>(m))-1) << (2 * nt_bufferlen);
@@ -469,7 +503,7 @@ void db_read(const char * filename)
                     {
                       while (datalen + sizeof(nt_buffer) > dataalloc)
                         {
-                          dataalloc += MEMCHUNK;
+                          dataalloc += memchunk;
                           datap = static_cast<char *>(xrealloc(datap, dataalloc));
                         }
 
@@ -480,25 +514,28 @@ void db_read(const char * filename)
                       nt_buffer = 0;
                     }
                 }
-              else if ((c != 10) && (c != 13))
+              else if ((c != new_line) && (c != carriage_return))
                 {
-                  if ((c >= 32) && (c <= 126))
+                  if ((c >= start_chars_range) && (c <= end_chars_range)) {
                     fprintf(stderr,
                             "\nError: Illegal character '%c' in sequence on line %u\n",
                             c,
                             lineno);
-                  else
+                  }
+                  else {
                     fprintf(stderr,
                             "\nError: Illegal character (ascii no %d) in sequence on line %u\n",
                             c,
                             lineno);
+                  }
                   exit(1);
                 }
-	    }
+            }
 
           line[0] = 0;
-          if (!fgets(line, LINEALLOC, fp))
+          if (fgets(line, linealloc, fp) == nullptr) {
             line[0] = 0;
+          }
           lineno++;
         }
 
@@ -514,8 +551,9 @@ void db_read(const char * filename)
 
       nucleotides += length;
 
-      if (length > longest)
+      if (length > longest) {
         longest = length;
+      }
 
 
       /* save remaining padded 64-bit value with nt's, if any */
@@ -524,7 +562,7 @@ void db_read(const char * filename)
         {
           while (datalen + sizeof(nt_buffer) > dataalloc)
             {
-              dataalloc += MEMCHUNK;
+              dataalloc += memchunk;
               datap = static_cast<char *>(xrealloc(datap, dataalloc));
             }
 
@@ -537,8 +575,9 @@ void db_read(const char * filename)
 
       sequences++;
 
-      if (is_regular)
+      if (is_regular) {
         progress_update(static_cast<uint64_t>(ftell(fp)));
+      }
     }
   progress_done();
 
@@ -546,25 +585,27 @@ void db_read(const char * filename)
 
   /* init zobrist hashing */
 
-  zobrist_init(longest + 2);  // add 2 for two insertions
+  // add 2 for two insertions
+  unsigned int zobrist_len = std::max(4 * longestheader, longest + 2);
+  zobrist_init(zobrist_len);
 
   /* set up hash to check for unique headers */
 
-  uint64_t hdrhashsize = 2 * sequences;
+  uint64_t hdrhashsize {2 * sequences};
 
-  seqinfo_t * * hdrhashtable =
+  auto * * hdrhashtable =
     static_cast<seqinfo_t **>(xmalloc(hdrhashsize * sizeof(seqinfo_t *)));
   memset(hdrhashtable, 0, hdrhashsize * sizeof(seqinfo_t *));
 
-  uint64_t duplicatedidentifiers = 0;
+  uint64_t duplicatedidentifiers {0};
 
   /* set up hash to check for unique sequences */
 
-  uint64_t seqhashsize = 2 * sequences;
+  uint64_t seqhashsize {2 * sequences};
 
-  seqinfo_t * * seqhashtable = nullptr;
+  seqinfo_t * * seqhashtable {nullptr};
 
-  if (opt_differences > 1)
+  if (p.opt_differences > 1)
     {
       seqhashtable =
         static_cast<seqinfo_t **>(xmalloc(seqhashsize * sizeof(seqinfo_t *)));
@@ -574,49 +615,52 @@ void db_read(const char * filename)
   /* create indices */
 
   seqindex = static_cast<seqinfo_t *>(xmalloc(sequences * sizeof(seqinfo_t)));
-  seqinfo_t * seqindex_p = seqindex;
+  seqinfo_t * seqindex_p {seqindex};
 
-  seqinfo_t * lastseq = nullptr;
+  seqinfo_t * lastseq {nullptr};
 
-  int presorted = 1;
+  bool presorted {true};
 
-  char * p = datap;
+  char * pl {datap};
   progress_init("Indexing database:", sequences);
-  for(uint64_t i=0; i<sequences; i++)
+  for(auto i = 0ULL; i < sequences; i++)
     {
       /* get line number */
-      unsigned int line_number = *(reinterpret_cast<unsigned int*>(p));
-      p += sizeof(unsigned int);
+      unsigned int line_number = *(reinterpret_cast<unsigned int*>(pl));
+      pl += sizeof(unsigned int);
 
       /* get header */
-      seqindex_p->header = p;
+      seqindex_p->header = pl;
       seqindex_p->headerlen = static_cast<int>(strlen(seqindex_p->header));
-      p += seqindex_p->headerlen + 1;
+      pl += seqindex_p->headerlen + 1;
 
       /* and sequence */
-      unsigned int seqlen = *(reinterpret_cast<unsigned int*>(p));
+      unsigned int seqlen = *(reinterpret_cast<unsigned int*>(pl));
       seqindex_p->seqlen = seqlen;
-      p += sizeof(unsigned int);
-      seqindex_p->seq = p;
-      p += nt_bytelength(seqlen);
+      pl += sizeof(unsigned int);
+      seqindex_p->seq = pl;
+      pl += nt_bytelength(seqlen);
 
       /* get amplicon abundance */
       find_abundance(seqindex_p, line_number);
 
       if ((seqindex_p->abundance_start == 0) &&
-          (seqindex_p->abundance_end == seqindex_p->headerlen))
+          (seqindex_p->abundance_end == seqindex_p->headerlen)) {
         fatal("Empty sequence identifier");
+      }
 
       /* check if the sequences are presorted by abundance and header */
 
-      if (presorted && lastseq)
+      if (presorted && (lastseq != nullptr))
         {
-          if (lastseq->abundance < seqindex_p->abundance)
-            presorted = 0;
+          if (lastseq->abundance < seqindex_p->abundance) {
+            presorted = false;
+          }
           else if (lastseq->abundance == seqindex_p->abundance)
             {
-              if (strcmp(lastseq->header, seqindex_p->header) > 0)
-                presorted = 0;
+              if (strcmp(lastseq->header, seqindex_p->header) > 0) {
+                presorted = false;
+              }
             }
         }
 
@@ -626,7 +670,8 @@ void db_read(const char * filename)
 
       /* find position and length of identifier in header */
 
-      int id_start, id_len;
+      int id_start {0};
+      int id_len {0};
 
       if (seqindex_p->abundance_start > 0)
         {
@@ -641,19 +686,21 @@ void db_read(const char * filename)
           id_len = seqindex_p->headerlen - seqindex_p->abundance_end;
         }
 
-      uint64_t hdrhash
-        = HASH(reinterpret_cast<unsigned char*>(seqindex_p->header + id_start),
-               static_cast<uint64_t>(id_len));
+      uint64_t hdrhash = zobrist_hash(reinterpret_cast<unsigned char*>
+                                      (seqindex_p->header + id_start),
+                                      4 * static_cast<unsigned int>(id_len));
+
       seqindex_p->hdrhash = hdrhash;
       uint64_t hdrhashindex = hdrhash % hdrhashsize;
 
-      seqinfo_t * hdrfound = nullptr;
+      seqinfo_t * hdrfound {nullptr};
 
-      while ((hdrfound = hdrhashtable[hdrhashindex]))
+      while ((hdrfound = hdrhashtable[hdrhashindex]) != nullptr)
         {
           if (hdrfound->hdrhash == hdrhash)
             {
-              int hit_id_start, hit_id_len;
+              int hit_id_start {0};
+              int hit_id_len {0};
 
               if (hdrfound->abundance_start > 0)
                 {
@@ -669,14 +716,15 @@ void db_read(const char * filename)
               if ((id_len == hit_id_len) &&
                   (strncmp(seqindex_p->header + id_start,
                            hdrfound->header + hit_id_start,
-                           static_cast<uint64_t>(id_len)) == 0))
+                           static_cast<uint64_t>(id_len)) == 0)) {
                 break;
+              }
             }
 
           hdrhashindex = (hdrhashindex + 1) % hdrhashsize;
         }
 
-      if (hdrfound)
+      if (hdrfound != nullptr)
         {
           duplicatedidentifiers++;
           fprintf(stderr, "\nError: Duplicated sequence identifier: %.*s\n\n",
@@ -692,39 +740,39 @@ void db_read(const char * filename)
                                          (seqindex_p->seq),
                                          seqindex_p->seqlen);
 
-      if (opt_differences > 1)
+      if (p.opt_differences > 1)
         {
           /* Check for duplicated sequences using hash table, */
           /* but only for d>1. Handled internally for d=1.    */
 
           uint64_t seqhashindex = seqindex_p->seqhash % seqhashsize;
-          seqinfo_t * seqfound = nullptr;
+          seqinfo_t * seqfound {nullptr};
 
-          while ((seqfound = seqhashtable[seqhashindex]))
+          while ((seqfound = seqhashtable[seqhashindex]) != nullptr)
             {
               if ((seqfound->seqhash == seqindex_p->seqhash) &&
                   (seqfound->seqlen == seqindex_p->seqlen) &&
                   (memcmp(seqfound->seq,
                           seqindex_p->seq,
-                          nt_bytelength(seqindex_p->seqlen)) == 0))
+                          nt_bytelength(seqindex_p->seqlen)) == 0)) {
                 break;
+              }
               seqhashindex = (seqhashindex + 1) % seqhashsize;
             }
 
-          if (seqfound)
+          if (seqfound != nullptr)
             {
               duplicates_found++;
               break;
             }
-          else
-            seqhashtable[seqhashindex] = seqindex_p;
+          seqhashtable[seqhashindex] = seqindex_p;
         }
 
       seqindex_p++;
       progress_update(i);
     }
 
-  if (duplicates_found)
+  if (duplicates_found != 0U)
     {
       fprintf(logfile,
               "\n\n"
@@ -739,7 +787,7 @@ void db_read(const char * filename)
 
   progress_done();
 
-  if (missingabundance)
+  if (missingabundance != 0)
     {
       fprintf(stderr,
               "\nError: Abundance annotations not found for %d sequences, starting on line %" PRIu64 ".\n"
@@ -764,7 +812,7 @@ void db_read(const char * filename)
 
   xfree(hdrhashtable);
 
-  if (seqhashtable)
+  if (seqhashtable != nullptr)
     {
       xfree(seqhashtable);
       seqhashtable = nullptr;
@@ -776,9 +824,9 @@ void db_qgrams_init()
   qgrams = static_cast<qgramvector_t *>
     (xmalloc(sequences * sizeof(qgramvector_t)));
 
-  seqinfo_t * seqindex_p = seqindex;
+  seqinfo_t * seqindex_p {seqindex};
   progress_init("Find qgram vects: ", sequences);
-  for(unsigned int i=0; i<sequences; i++)
+  for(auto i = 0U; i < sequences; i++)
     {
       /* find qgrams */
       findqgrams(reinterpret_cast<unsigned char*>(seqindex_p->seq),
@@ -795,27 +843,27 @@ void db_qgrams_done()
   xfree(qgrams);
 }
 
-unsigned int db_getsequencecount()
+auto db_getsequencecount() -> unsigned int
 {
   return sequences;
 }
 
-uint64_t db_getnucleotidecount()
+auto db_getnucleotidecount() -> uint64_t
 {
   return nucleotides;
 }
 
-unsigned int db_getlongestsequence()
+auto db_getlongestsequence() -> unsigned int
 {
   return longest;
 }
 
-uint64_t db_gethash(uint64_t seqno)
+auto db_gethash(uint64_t seqno) -> uint64_t
 {
   return seqindex[seqno].seqhash;
 }
 
-char * db_getsequence(uint64_t seqno)
+auto db_getsequence(uint64_t seqno) -> char *
 {
   return seqindex[seqno].seq;
 }
@@ -828,17 +876,17 @@ void db_getsequenceandlength(uint64_t seqno,
   *length = seqindex[seqno].seqlen;
 }
 
-unsigned int db_getsequencelen(uint64_t seqno)
+auto db_getsequencelen(uint64_t seqno) -> unsigned int
 {
   return seqindex[seqno].seqlen;
 }
 
-char * db_getheader(uint64_t seqno)
+auto db_getheader(uint64_t seqno) -> char *
 {
   return seqindex[seqno].header;
 }
 
-uint64_t db_getabundance(uint64_t seqno)
+auto db_getabundance(uint64_t seqno) -> uint64_t
 {
   return seqindex[seqno].abundance;
 }
@@ -847,42 +895,49 @@ void db_free()
 {
   zobrist_exit();
 
-  if (datap)
+  if (datap != nullptr) {
     xfree(datap);
-  if (seqindex)
+  }
+  if (seqindex != nullptr) {
     xfree(seqindex);
+  }
 }
 
 void db_fprintseq(FILE * fp, unsigned int a, unsigned int width)
 {
+  constexpr int default_length {1025};
   char * seq = db_getsequence(a);
   unsigned int len = db_getsequencelen(a);
-  char buffer[1025];
-  char * buf;
+  char buffer[default_length];
+  char * buf {nullptr};
 
-  if (len < 1025)
+  // buf = len < default_length ? buffer : static_cast<char*>(xmalloc(len+1));
+  if (len < default_length) {
     buf = buffer;
-  else
+  }
+  else {
     buf = static_cast<char*>(xmalloc(len+1));
+  }
 
-  for(unsigned int i = 0; i < len; i++)
+  for(auto i = 0U; i < len; i++) {
     buf[i] = sym_nt[1 + nt_extract(seq, i)];
+  }
   buf[len] = 0;
 
-  if (width < 1)
+  if (width < 1) {
     fprintf(fp, "%.*s\n", len, buf);
-  else
-    {
-      unsigned int rest = len;
-      for(unsigned int i = 0; i < len; i += width)
-        {
-          fprintf(fp, "%.*s\n", MIN(rest, width), buf+i);
-          rest -= width;
-        }
+  }
+  else {
+    auto rest = len;
+    for(auto i = 0U; i < len; i += width) {
+      fprintf(fp, "%.*s\n", std::min(rest, width), buf+i);
+      rest -= width;
     }
+  }
 
-  if (len >= 1025)
+  if (len >= default_length) {
     xfree(buf);
+  }
 }
 
 
@@ -907,11 +962,12 @@ seqinfo_t * db_getseqinfo(uint64_t seqno)
 
 void db_putseq(int64_t seqno)
 {
-  char * seq;
-  int64_t len;
+  char * seq {nullptr};
+  int64_t len {0};
   db_getsequenceandlength(seqno, & seq, & len);
-  for(int i=0; i<len; i++)
+  for(auto i = 0LL; i < len; i++) {
     putchar(sym_nt[1+nt_extract(seq, i)]);
+  }
 }
 
 #endif
