@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2017 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,86 +23,77 @@
 
 #include "swarm.h"
 
-//#define REVCOMP
-
 struct bucket
 {
-  unsigned long hash;
+  uint64_t hash;
   unsigned int seqno_first;
   unsigned int seqno_last;
-  unsigned long mass;
+  uint64_t mass;
   unsigned int size;
   unsigned int singletons;
 };
 
-int derep_compare(const void * a, const void * b)
+auto derep_compare(const void * a, const void * b) -> int;
+
+auto derep_compare(const void * a, const void * b) -> int
 {
-  struct bucket * x = (struct bucket *) a;
-  struct bucket * y = (struct bucket *) b;
+  const auto * x = static_cast<const struct bucket *>(a);
+  const auto * y = static_cast<const struct bucket *>(b);
+  int status {0};
 
   /* highest abundance first, otherwise keep order */
 
-  if (x->mass < y->mass)
-    return +1;
-  else if (x->mass > y->mass)
-    return -1;
-  else
-    {
-      if (x->seqno_first < y->seqno_first)
-        return -1;
-      else if (x->seqno_first > y->seqno_first)
-        return +1;
-      else
-        return 0;
+  if (x->mass < y->mass) {
+    status = +1;
+  }
+  else if (x->mass > y->mass) {
+    status = -1;
+  }
+  else {
+    if (x->seqno_first < y->seqno_first) {
+      status = -1;
     }
+    else if (x->seqno_first > y->seqno_first) {
+      status = +1;
+    }
+    else {
+      status = 0;
+    }
+  }
+
+  return status;
 }
 
-#ifdef REVCOMP
-char map_complement[5] = { 0, 4, 3, 2, 1 };
-
-void reverse_complement(char * rc, char * seq, long len)
-{
-  /* Write the reverse complementary sequence to rc.
-     The memory for rc must be long enough for the rc of the sequence
-     (identical to the length of seq + 1). */
-
-  for(long i=0; i<len; i++)
-    rc[i] = map_complement[(int)(seq[len-1-i])];
-  rc[len] = 0;
-}
-#endif
-
-void dereplicate()
+void dereplicate(struct Parameters const & p)
 {
   /* adjust size of hash table for 2/3 fill rate */
-  long dbsequencecount = db_getsequencecount();
-  long hashtablesize = 1;
-  while (1.0 * dbsequencecount / hashtablesize > 0.7)
+  constexpr unsigned int hashfillpct {70};
+  constexpr unsigned int one_hundred_pct {100};
+  uint64_t dbsequencecount = db_getsequencecount();
+  uint64_t hashtablesize {1};
+  // db seqs > 70% hash table size (avoid division to keep working with ints)
+  while (one_hundred_pct * dbsequencecount > hashfillpct * hashtablesize) {
     hashtablesize <<= 1;
-  int hash_mask = hashtablesize - 1;
+  }
+  uint64_t derep_hash_mask = hashtablesize - 1;
 
-  struct bucket * hashtable =
-    (struct bucket *) xmalloc(sizeof(bucket) * hashtablesize);
+  auto * hashtable =
+    static_cast<struct bucket *>(xmalloc(sizeof(bucket) * hashtablesize));
 
   memset(hashtable, 0, sizeof(bucket) * hashtablesize);
 
-  long swarmcount = 0;
-  unsigned long maxmass = 0;
+  uint64_t swarmcount = 0;
+  uint64_t maxmass = 0;
   unsigned int maxsize = 0;
 
   /* alloc and init table of links to other sequences in cluster */
-  unsigned int * nextseqtab = (unsigned int *)
-    xmalloc(sizeof(unsigned int) * dbsequencecount);
+  auto * nextseqtab = static_cast<unsigned int *>
+    (xmalloc(sizeof(unsigned int) * dbsequencecount));
   memset(nextseqtab, 0, sizeof(unsigned int) * dbsequencecount);
 
-#ifdef REVCOMP
-  /* allocate memory for reverse complementary sequence */
-  char * rc_seq = (char*) xmalloc(db_getlongestsequence() + 1);
-#endif
-  
   progress_init("Dereplicating:    ", dbsequencecount);
 
-  for(long i=0; i<dbsequencecount; i++)
+  for(auto i = 0U; i < dbsequencecount; i++)
     {
       unsigned int seqlen = db_getsequencelen(i);
       char * seq = db_getsequence(i);
@@ -115,14 +106,18 @@ void dereplicate()
         collision when the number of sequences is about 5e9.
       */
 
-      unsigned long hash = CityHash64(seq, seqlen);
-      unsigned long j = hash & hash_mask;
+      uint64_t hash = zobrist_hash(reinterpret_cast<unsigned char *>(seq),
+                                   seqlen);
+
+      uint64_t j = hash & derep_hash_mask;
       struct bucket * bp = hashtable + j;
-      
-      while ((bp->mass) &&
+
+      while (((bp->mass) != 0U) &&
              ((bp->hash != hash) ||
               (seqlen != db_getsequencelen(bp->seqno_first)) ||
-              (strcmp(seq, db_getsequence(bp->seqno_first)))))
+              (memcmp(seq,
+                      db_getsequence(bp->seqno_first),
+                      nt_bytelength(seqlen)) != 0)))
         {
           bp++;
           j++;
@@ -133,42 +128,9 @@ void dereplicate()
             }
         }
 
-#ifdef REVCOMP
-      if (! bp->mass)
-        {
-          /* no match on plus strand */
-          /* check minus strand as well */
+      uint64_t ab = db_getabundance(i);
 
-          reverse_complement(rc_seq, seq, seqlen);
-          unsigned long rc_hash = CityHash64(rc_seq, seqlen);
-          struct bucket * rc_bp = hashtable + rc_hash % hashtablesize;
-          unsigned long k = rc_hash & hash_mask;
-          
-          while ((rc_bp->mass) &&
-                 ((rc_bp->hash != rc_hash) ||
-                  (seqlen != db_getsequencelen(rc_bp->seqno_first)) ||
-                  (strcmp(rc_seq, db_getsequence(rc_bp->seqno_first)))))
-            {
-              rc_bp++;
-              k++;
-              if (rc_bp >= hashtable + hashtablesize)
-                {
-                  rc_bp = hashtable;
-                  k++;
-                }
-            }
-
-          if (rc_bp->mass)
-            {
-              bp = rc_bp;
-              j = k;
-            }
-        }
-#endif
-
-      long ab = db_getabundance(i);
-
-      if (bp->mass)
+      if ((bp->mass) != 0U)
         {
           /* at least one identical sequence already */
           nextseqtab[bp->seqno_last] = i;
@@ -187,23 +149,22 @@ void dereplicate()
       bp->seqno_last = i;
       bp->mass += ab;
 
-      if (ab == 1)
+      if (ab == 1) {
         bp->singletons++;
+      }
 
-      if (bp->mass > maxmass)
+      if (bp->mass > maxmass) {
         maxmass = bp->mass;
+      }
 
-      if (bp->size > maxsize)
+      if (bp->size > maxsize) {
         maxsize = bp->size;
+      }
 
       progress_update(i);
     }
   progress_done();
 
-#ifdef REVCOMP
-  free(rc_seq);
-#endif
-  
   progress_init("Sorting:          ", 1);
   qsort(hashtable, hashtablesize, sizeof(bucket), derep_compare);
   progress_done();
@@ -213,47 +174,53 @@ void dereplicate()
 
   progress_init("Writing swarms:   ", swarmcount);
 
-  if (opt_mothur)
-    fprintf(outfile, "swarm_%ld\t%ld", opt_differences, swarmcount);
+  if (opt_mothur) {
+    fprintf(outfile, "swarm_%" PRId64 "\t%" PRIu64, p.opt_differences, swarmcount);
+  }
 
-  for(int i = 0; i < swarmcount; i++)
+  for(auto i = 0U; i < swarmcount; i++)
     {
-      int seed = hashtable[i].seqno_first;
-      if (opt_mothur)
+      unsigned int seed = hashtable[i].seqno_first;
+      if (opt_mothur) {
         fputc('\t', outfile);
+      }
       fprint_id(outfile, seed);
-      int a = nextseqtab[seed];
+      unsigned int a = nextseqtab[seed];
 
-      while (a)
+      while (a != 0U)
         {
-          if (opt_mothur)
+          if (opt_mothur) {
             fputc(',', outfile);
-          else
-            fputc(SEPCHAR, outfile);
+          }
+          else {
+            fputc(sepchar, outfile);
+          }
           fprint_id(outfile, a);
           a = nextseqtab[a];
         }
-      
-      if (!opt_mothur)
+
+      if (!opt_mothur) {
         fputc('\n', outfile);
+      }
 
       progress_update(i+1);
     }
 
-  if (opt_mothur)
+  if (opt_mothur) {
     fputc('\n', outfile);
-  
+  }
+
   progress_done();
 
 
   /* dump seeds in fasta format with sum of abundances */
 
-  if (opt_seeds)
+  if (! opt_seeds.empty())
     {
       progress_init("Writing seeds:    ", swarmcount);
-      for(int i=0; i < swarmcount; i++)
+      for(auto i = 0U; i < swarmcount; i++)
         {
-          int seed = hashtable[i].seqno_first;
+          unsigned int seed = hashtable[i].seqno_first;
           fprintf(fp_seeds, ">");
           fprint_id_with_new_abundance(fp_seeds, seed, hashtable[i].mass);
           fprintf(fp_seeds, "\n");
@@ -265,37 +232,37 @@ void dereplicate()
 
   /* output swarm in uclust format */
 
-  if (uclustfile)
+  if (uclustfile != nullptr)
     {
       progress_init("Writing UCLUST:   ", swarmcount);
 
-      for(unsigned int swarmid = 0; swarmid < swarmcount ; swarmid++)
+      for(auto swarmid = 0U; swarmid < swarmcount; swarmid++)
         {
           struct bucket * bp = hashtable + swarmid;
-          
-          int seed = bp->seqno_first;
+
+          unsigned int seed = bp->seqno_first;
 
           fprintf(uclustfile, "C\t%u\t%u\t*\t*\t*\t*\t*\t",
                   swarmid,
                   bp->size);
           fprint_id(uclustfile, seed);
           fprintf(uclustfile, "\t*\n");
-          
-          fprintf(uclustfile, "S\t%u\t%lu\t*\t*\t*\t*\t*\t",
+
+          fprintf(uclustfile, "S\t%u\t%u\t*\t*\t*\t*\t*\t",
                   swarmid,
                   db_getsequencelen(seed));
           fprint_id(uclustfile, seed);
           fprintf(uclustfile, "\t*\n");
-          
-          int a = nextseqtab[seed];
 
-          while (a)
+          unsigned int a = nextseqtab[seed];
+
+          while (a != 0U)
             {
               fprintf(uclustfile,
-                      "H\t%u\t%lu\t%.1f\t+\t0\t0\t%s\t",
+                      "H\t%u\t%u\t%.1f\t+\t0\t0\t%s\t",
                       swarmid,
                       db_getsequencelen(a),
-                      100.0, 
+                      100.0,
                       "=");
               fprint_id(uclustfile, a);
               fprintf(uclustfile, "\t");
@@ -303,7 +270,7 @@ void dereplicate()
               fprintf(uclustfile, "\n");
               a = nextseqtab[a];
             }
-          
+
           progress_update(swarmid+1);
         }
       progress_done();
@@ -311,21 +278,21 @@ void dereplicate()
 
   /* output internal structure to file */
 
-  if (opt_internal_structure)
+  if (! p.opt_internal_structure.empty())
     {
       progress_init("Writing structure:", swarmcount);
-      
-      for(long i = 0; i < swarmcount; i++)
+
+      for(uint64_t i = 0; i < swarmcount; i++)
         {
           struct bucket * sp = hashtable + i;
-          long seed = sp->seqno_first;
-          int a = nextseqtab[seed];
-          while (a)
+          uint64_t seed = sp->seqno_first;
+          unsigned int a = nextseqtab[seed];
+          while (a != 0U)
             {
               fprint_id_noabundance(internal_structure_file, seed);
               fprintf(internal_structure_file, "\t");
               fprint_id_noabundance(internal_structure_file, a);
-              fprintf(internal_structure_file, "\t%d\t%ld\t%d\n", 0, i+1, 0);
+              fprintf(internal_structure_file, "\t%d\t%" PRIu64 "\t%d\n", 0, i+1, 0);
               a = nextseqtab[a];
             }
           progress_update(i);
@@ -335,15 +302,15 @@ void dereplicate()
 
   /* output statistics to file */
 
-  if (statsfile)
+  if (statsfile != nullptr)
     {
       progress_init("Writing stats:    ", swarmcount);
-      for(long i = 0; i < swarmcount; i++)
+      for(auto i = 0ULL; i < swarmcount; i++)
         {
           struct bucket * sp = hashtable + i;
-          fprintf(statsfile, "%u\t%lu\t", sp->size, sp->mass);
+          fprintf(statsfile, "%u\t%" PRIu64 "\t", sp->size, sp->mass);
           fprint_id_noabundance(statsfile, sp->seqno_first);
-          fprintf(statsfile, "\t%lu\t%u\t%u\t%u\n", 
+          fprintf(statsfile, "\t%" PRIu64 "\t%u\t%u\t%u\n",
                   db_getabundance(sp->seqno_first),
                   sp->singletons, 0U, 0U);
           progress_update(i);
@@ -353,10 +320,10 @@ void dereplicate()
 
 
   fprintf(logfile, "\n");
-  fprintf(logfile, "Number of swarms:  %ld\n", swarmcount);
+  fprintf(logfile, "Number of swarms:  %" PRIu64 "\n", swarmcount);
   fprintf(logfile, "Largest swarm:     %u\n", maxsize);
-  fprintf(logfile, "Heaviest swarm:    %lu\n", maxmass);
+  fprintf(logfile, "Heaviest swarm:    %" PRIu64 "\n", maxmass);
 
-  free(nextseqtab);
-  free(hashtable);
+  xfree(nextseqtab);
+  xfree(hashtable);
 }
