@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2019 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,28 +23,34 @@
 
 #include "swarm.h"
 
-#define HASHFILLPCT 70
-
-uint64_t hash_mask;
-unsigned char * hash_occupied = nullptr;
-uint64_t * hash_values = nullptr;
-unsigned int * hash_data = nullptr;
-uint64_t hash_tablesize = 0;
+uint64_t hash_mask {0};
+unsigned char * hash_occupied {nullptr};
+uint64_t * hash_values {nullptr};
+unsigned int * hash_data {nullptr};
+uint64_t hash_tablesize {0};
 
 void hash_zap()
 {
-  memset(hash_occupied, 0, (hash_tablesize + 63) / 8);
+  constexpr int padding {63};  // make sure our final value is >= 64 / 8
+  constexpr int convert_to_bytes {8};
+  memset(hash_occupied, 0, (hash_tablesize + padding) / convert_to_bytes);
 }
 
 void hash_alloc(uint64_t amplicons)
 {
+  constexpr unsigned int hashfillpct {70};
+  constexpr unsigned int one_hundred_pct {100};
+  constexpr int padding {63};  // make sure our final value is >= 64 / 8
+  constexpr int convert_to_bytes {8};
   hash_tablesize = 1;
-  while (100 * amplicons > HASHFILLPCT * hash_tablesize)
+  // amplicons > 70% hash table size (avoid division to keep working with ints)
+  while (one_hundred_pct * amplicons > hashfillpct * hash_tablesize) {
     hash_tablesize <<= 1;
+  }
   hash_mask = hash_tablesize - 1;
 
   hash_occupied =
-    static_cast<unsigned char *>(xmalloc((hash_tablesize + 63) / 8));
+    static_cast<unsigned char *>(xmalloc((hash_tablesize + padding) / convert_to_bytes));
   hash_zap();
 
   hash_values =
