@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2019 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,13 +23,6 @@
 
 #include "swarm.h"
 
-int64_t SCORELIMIT_7 = 0;
-int64_t SCORELIMIT_8;
-int64_t SCORELIMIT_16;
-int64_t SCORELIMIT_32;
-int64_t SCORELIMIT_63;
-char BIAS;
-
 unsigned char * score_matrix_8 = nullptr;
 unsigned short * score_matrix_16 = nullptr;
 int64_t * score_matrix_63 = nullptr;
@@ -43,17 +36,17 @@ void score_matrix_read();
 void score_matrix_dump()
 {
   fprintf(logfile, "     ");
-  for(int i=0; i<16; i++)
+  for(auto i = 0; i < 16; i++)
     fprintf(logfile, "%2d", i);
   fprintf(logfile, "\n");
   fprintf(logfile, "     ");
-  for(int i=0; i<16; i++)
+  for(auto i = 0; i < 16; i++)
     fprintf(logfile, " %c", sym_nt[i]);
   fprintf(logfile, "\n");
-  for(int i=0; i<16; i++)
+  for(auto i = 0; i < 16; i++)
   {
     fprintf(logfile, "%2d %c ", i, sym_nt[i]);
-    for(int j=0; j<16; j++)
+    for(auto j = 0; j < 16; j++)
       {
         fprintf(logfile, "%2" PRId64, score_matrix_63[(i<<5) + j]);
       }
@@ -65,39 +58,38 @@ void score_matrix_dump()
 
 void score_matrix_read()
 {
-  int a, b;
-  int64_t sc, lo, hi;
+  constexpr int cells {32};
+  constexpr long long int one_thousand {1000};
+  constexpr unsigned int multiplier {5};
+  long long int sc {0};
+  long long int hi {-one_thousand};
+  long long int lo {one_thousand};
 
-  score_matrix_8 = static_cast<unsigned char*>(xmalloc(32*32*sizeof(char)));
-  score_matrix_16 = static_cast<unsigned short*>(xmalloc(32*32*sizeof(short)));
-  score_matrix_63 = static_cast<int64_t *>(xmalloc(32*32*sizeof(int64_t)));
+  score_matrix_8 = static_cast<unsigned char *>(xmalloc(cells * cells * sizeof(char)));
+  score_matrix_16 = static_cast<unsigned short *>(xmalloc(cells * cells * sizeof(short)));
+  score_matrix_63 = static_cast<int64_t *>(xmalloc(cells * cells * sizeof(int64_t)));
 
-  hi = -1000;
-  lo = 1000;
-
-  for(a=0;a<16;a++)
-    for(b=0;b<16;b++)
-    {
-      sc = ((a==b)&&(a>0)&&(b>0)) ? 0 : penalty_mismatch;
+  for(auto a = 0; a < cells / 2; a++) {
+    for(auto b = 0; b < cells / 2; b++) {
+      sc = ((a == b) && (a > 0) && (b > 0)) ? 0 : penalty_mismatch;
       // sc = (a==b) ? matchscore : mismatchscore;
-      if (sc < lo)
+      if (sc < lo) {
         lo = sc;
-      if (sc > hi)
+      }
+      if (sc > hi) {
         hi = sc;
-      score_matrix_63[(a<<5) + b] = sc;
+      }
+      score_matrix_63[(a << multiplier) + b] = sc;
     }
+  }
 
-
-  SCORELIMIT_8  = 256 - hi;
-  SCORELIMIT_16 = 65536 - hi;
-
-  for(a=0;a<32;a++)
-    for(b=0;b<32;b++)
-    {
-      sc = score_matrix_63[(a<<5) + b];
-      score_matrix_8[(a<<5) + b] = static_cast<unsigned char>(sc);
-      score_matrix_16[(a<<5) + b] = static_cast<unsigned short>(sc);
+  for(auto a = 0; a < cells; a++) {
+    for(auto b = 0; b < cells; b++) {
+      sc = score_matrix_63[(a << multiplier) + b];
+      score_matrix_8[(a << multiplier) + b] = static_cast<unsigned char>(sc);
+      score_matrix_16[(a << multiplier) + b] = static_cast<unsigned short>(sc);
     }
+  }
 }
 
 void score_matrix_init()
