@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2019 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -28,15 +28,18 @@ void finishop(char ** cigarendp, char * op, int * count);
 
 void pushop(char newop, char ** cigarendp, char * op, int * count)
 {
-  if (newop == *op)
+  constexpr unsigned int buffer_length {25};
+
+  if (newop == *op) {
     (*count)++;
+  }
   else
   {
     *--*cigarendp = *op;
     if (*count > 1)
     {
-      char buf[25];
-      int len = snprintf(buf, 25, "%d", *count);
+      char buf[buffer_length];
+      int len = snprintf(buf, buffer_length, "%d", *count);
       assert(len >= 0);
       *cigarendp -= len;
       memcpy(*cigarendp, buf, static_cast<size_t>(len));
@@ -48,13 +51,15 @@ void pushop(char newop, char ** cigarendp, char * op, int * count)
 
 void finishop(char ** cigarendp, char * op, int * count)
 {
-  if ((op) && (count))
+  constexpr unsigned int buffer_length {25};
+
+  if ((op != nullptr) && (count != nullptr))
   {
     *--*cigarendp = *op;
     if (*count > 1)
     {
-      char buf[25];
-      int len = snprintf(buf, 25, "%d", *count);
+      char buf[buffer_length];
+      int len = snprintf(buf, buffer_length, "%d", *count);
       assert(len >= 0);
       *cigarendp -= len;
       memcpy(*cigarendp, buf, static_cast<size_t>(len));
@@ -129,39 +134,39 @@ void nw(char * dseq,
   /* dir must point to at least qlen*dlen bytes of allocated memory
      hearray must point to at least 2*qlen longs of allocated memory
      (8*qlen bytes) */
+  constexpr unsigned int multiplier {5};
 
-  int64_t n, e;
+  int64_t n {0};
+  int64_t e {0};
 
   memset(dir, 0, static_cast<size_t>(qlen * dlen));
 
-  int64_t i, j;
-
-  for(i=0; i<qlen; i++)
+  for(auto i = 0L; i < qlen; i++)
     {
-      hearray[2*i]   = 1 * gapopen + (i+1) * gapextend; // H (N)
-      hearray[2*i+1] = 2 * gapopen + (i+2) * gapextend; // E
+      hearray[2 * i]     = 1 * gapopen + (i + 1) * gapextend; // H (N)
+      hearray[2 * i + 1] = 2 * gapopen + (i + 2) * gapextend; // E
     }
 
-  for(j=0; j<dlen; j++)
+  for(auto j = 0L; j < dlen; j++)
     {
-      int64_t *hep;
+      int64_t * hep {nullptr};
       hep = hearray;
       int64_t f = 2 * gapopen + (j+2) * gapextend;
       int64_t h = (j == 0) ? 0 : (gapopen + j * gapextend);
 
-      for(i=0; i<qlen; i++)
+      for(auto i = 0L; i < qlen; i++)
         {
-          int64_t index = qlen*j+i;
+          int64_t index = qlen * j + i;
 
           n = *hep;
           e = *(hep+1);
           h += score_matrix
-            [((nt_extract(dseq, static_cast<uint64_t>(j)) + 1) << 5)
+            [((nt_extract(dseq, static_cast<uint64_t>(j)) + 1) << multiplier)
              +(nt_extract(qseq, static_cast<uint64_t>(i)) + 1)];
 
           dir[index] |= (f < h ? maskup : 0);
-          h = MIN(h, f);
-          h = MIN(h, e);
+          h = std::min(h, f);
+          h = std::min(h, e);
           dir[index] |= (e == h ? maskleft : 0);
 
           *hep = h;
@@ -172,8 +177,8 @@ void nw(char * dseq,
 
           dir[index] |= (f < h ? maskextup : 0);
           dir[index] |= (e < h ? maskextleft : 0);
-          f = MIN(h,f);
-          e = MIN(h,e);
+          f = std::min(h,f);
+          e = std::min(h,e);
 
           *(hep+1) = e;
           h = n;
@@ -181,24 +186,23 @@ void nw(char * dseq,
         }
     }
 
-  int64_t dist = hearray[2*qlen-2];
+  int64_t dist = hearray[2 * qlen - 2];
 
   /* backtrack: count differences and save alignment in cigar string */
-
-  int64_t score = 0;
-  int64_t alength = 0;
-  int64_t matches = 0;
+  int64_t score {0};
+  int64_t alength {0};
+  int64_t matches {0};
 
   char * cigar = static_cast<char *>(xmalloc
                                      (static_cast<size_t>(qlen + dlen + 1)));
-  char * cigarend = cigar + qlen + dlen + 1;
+  char * cigarend {cigar + qlen + dlen + 1};
 
-  char op = 0;
-  int count = 0;
+  char op {0};
+  int count {0};
   *(--cigarend) = 0;
 
-  i = qlen;
-  j = dlen;
+  int64_t i = qlen;
+  int64_t j = dlen;
 
   while ((i>0) && (j>0))
     {
@@ -206,43 +210,46 @@ void nw(char * dseq,
 
       alength++;
 
-      if ((op == 'I') && (d & maskextleft))
+      if ((op == 'I') && ((d & maskextleft) != 0))
         {
           score += gapextend;
           j--;
           pushop('I', &cigarend, &op, &count);
         }
-      else if ((op == 'D') && (d & maskextup))
+      else if ((op == 'D') && ((d & maskextup) != 0))
         {
           score += gapextend;
           i--;
           pushop('D', &cigarend, &op, &count);
         }
-      else if (d & maskleft)
+      else if ((d & maskleft) != 0)
         {
           score += gapextend;
-          if (op != 'I')
+          if (op != 'I') {
             score += gapopen;
+          }
           j--;
           pushop('I', &cigarend, &op, &count);
         }
-      else if (d & maskup)
+      else if ((d & maskup) != 0)
         {
           score += gapextend;
-          if (op != 'D')
+          if (op != 'D') {
             score += gapopen;
+          }
           i--;
           pushop('D', &cigarend, &op, &count);
         }
       else
         {
           score += score_matrix
-            [((nt_extract(dseq, static_cast<uint64_t>(j - 1)) + 1) << 5)
+            [((nt_extract(dseq, static_cast<uint64_t>(j - 1)) + 1) << multiplier)
              +(nt_extract(qseq, static_cast<uint64_t>(i - 1)) + 1)];
 
           if (nt_extract(qseq, static_cast<uint64_t>(i - 1)) ==
-              nt_extract(dseq, static_cast<uint64_t>(j - 1)))
+              nt_extract(dseq, static_cast<uint64_t>(j - 1))) {
             matches++;
+          }
           i--;
           j--;
           pushop('M', &cigarend, &op, &count);
@@ -253,8 +260,9 @@ void nw(char * dseq,
     {
       alength++;
       score += gapextend;
-      if (op != 'D')
+      if (op != 'D') {
         score += gapopen;
+      }
       i--;
       pushop('D', &cigarend, &op, &count);
     }
@@ -263,8 +271,9 @@ void nw(char * dseq,
     {
       alength++;
       score += gapextend;
-      if (op != 'I')
+      if (op != 'I') {
         score += gapopen;
+      }
       j--;
       pushop('I', &cigarend, &op, &count);
     }
@@ -273,7 +282,7 @@ void nw(char * dseq,
 
   /* move and reallocate cigar */
 
-  size_t cigaralloc = static_cast<size_t>(cigar + qlen + dlen - cigarend + 1);
+  auto cigaralloc = static_cast<size_t>(cigar + qlen + dlen - cigarend + 1);
   memmove(cigar, cigarend, cigaralloc);
   cigar = static_cast<char*>(xrealloc(cigar, cigaralloc));
 
@@ -287,8 +296,12 @@ void nw(char * dseq,
 #if 0
   if (score != dist)
   {
-    fprintf(stderr, "WARNING: Error with query no %" PRIu64 " and db sequence no %" PRIu64 ":\n", queryno, dbseqno);
-    fprintf(stderr, "Initial and recomputed alignment score disagreement: %" PRId64 " %" PRId64 "\n", dist, score);
+    fprintf(stderr,
+            "WARNING: Error with query no %" PRIu64 " and db sequence no %" PRIu64 ":\n",
+            queryno, dbseqno);
+    fprintf(stderr,
+            "Initial and recomputed alignment score disagreement: %" PRId64 " %" PRId64 "\n",
+            dist, score);
     fprintf(stderr, "Alignment: %s\n", cigar);
   }
 #else
