@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2019 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,14 +23,16 @@
 
 #include "swarm.h"
 
-#define CHANNELS 8
-#define CDEPTH 4
+constexpr unsigned int channels {8};
+constexpr unsigned int cdepth {4};
+
+/* uses 8 unsigned 16-bit values */
 
 #ifdef __aarch64__
 
-typedef int16x8_t VECTORTYPE;
+using VECTORTYPE = int16x8_t;
 
-#define CAST_VECTOR_p(x) reinterpret_cast<VECTORTYPE *>(x)
+#define CAST_VECTOR_p(x) (reinterpret_cast<VECTORTYPE *>(x))
 
 const uint16x8_t neon_mask =
   {0x0003, 0x000c, 0x0030, 0x00c0, 0x0300, 0x0c00, 0x3000, 0xc000};
@@ -49,7 +51,7 @@ const uint16x8_t neon_mask =
 #define v_merge_hi_64(a, b) vreinterpretq_s16_s64(vcombine_s64 \
           (vget_high_s64(vreinterpretq_s64_s16(a)), \
            vget_high_s64(vreinterpretq_s64_s16(b))))
-#define v_min(a, b) vminq_s16((a), (b))
+#define v_min(a, b) vminq_u16((a), (b))
 #define v_add(a, b) vqaddq_u16((a), (b))
 #define v_sub(a, b) vqsubq_u16((a), (b))
 #define v_dup(a) vdupq_n_s16(a)
@@ -57,14 +59,13 @@ const uint16x8_t neon_mask =
 #define v_and(a, b) vandq_u16((a), (b))
 #define v_xor(a, b) veorq_u16((a), (b))
 #define v_shift_left(a) vextq_u16((v_zero), (a), 7)
-#define v_mask_gt(a, b) vaddvq_u16(vandq_u16((vcgtq_s16((a), (b))), neon_mask))
 #define v_mask_eq(a, b) vaddvq_u16(vandq_u16((vceqq_s16((a), (b))), neon_mask))
 
 #elif defined __x86_64__
 
-typedef __m128i VECTORTYPE;
+using VECTORTYPE = __m128i;
 
-#define CAST_VECTOR_p(x) reinterpret_cast<VECTORTYPE *>(x)
+#define CAST_VECTOR_p(x) (reinterpret_cast<VECTORTYPE *>(x))
 
 #define v_load(a) _mm_load_si128(CAST_VECTOR_p(a))
 #define v_store(a, b) _mm_store_si128(CAST_VECTOR_p(a), (b))
@@ -74,7 +75,7 @@ typedef __m128i VECTORTYPE;
 #define v_merge_hi_32(a, b) _mm_unpackhi_epi32((a),(b))
 #define v_merge_lo_64(a, b) _mm_unpacklo_epi64((a),(b))
 #define v_merge_hi_64(a, b) _mm_unpackhi_epi64((a),(b))
-#define v_min(a, b) _mm_min_epi16((a), (b))
+#define v_min(a, b) _mm_subs_epu16((a), _mm_subs_epu16((a), (b)))
 #define v_add(a, b) _mm_adds_epu16((a), (b))
 #define v_sub(a, b) _mm_subs_epu16((a), (b))
 #define v_dup(a) _mm_set1_epi16(a)
@@ -82,12 +83,12 @@ typedef __m128i VECTORTYPE;
 #define v_and(a, b) _mm_and_si128((a), (b))
 #define v_xor(a, b) _mm_xor_si128((a), (b))
 #define v_shift_left(a) _mm_slli_si128((a), 2)
-#define v_mask_gt(a, b) static_cast<unsigned short>(_mm_movemask_epi8(_mm_cmpgt_epi16((a), (b))))
-#define v_mask_eq(a, b) static_cast<unsigned short>(_mm_movemask_epi8(_mm_cmpeq_epi16((a), (b))))
+#define v_mask_eq(a, b) static_cast<unsigned short> \
+  (_mm_movemask_epi8(_mm_cmpeq_epi16((a), (b))))
 
 #elif defined __PPC__
 
-typedef vector unsigned short VECTORTYPE;
+using VECTORTYPE = vector unsigned short;
 
 #define CAST_VECTOR_p(x) reinterpret_cast<VECTORTYPE *>(x)
 
@@ -117,8 +118,7 @@ const vector unsigned char perm_bits =
 #define v_merge_hi_64(a, b) (VECTORTYPE) vec_perm((vector long long)(a), \
                                                   (vector long long)(b), \
                                                   perm_merge_long_high)
-#define v_min(a, b) (VECTORTYPE) vec_min((vector signed short) (a),     \
-                                         (vector signed short) (b))
+#define v_min(a, b) vec_min((a), (b))
 #define v_add(a, b) vec_adds((a), (b))
 #define v_sub(a, b) vec_subs((a), (b))
 #define v_dup(a) vec_splats((unsigned short)(a));
@@ -126,8 +126,6 @@ const vector unsigned char perm_bits =
 #define v_and(a, b) vec_and((a), (b))
 #define v_xor(a, b) vec_xor((a), (b))
 #define v_shift_left(a) vec_sld((a), v_zero, 2)
-#define v_mask_gt(a, b) ((vector unsigned short) \
-  vec_vbpermq((vector unsigned char) vec_cmpgt((a), (b)), perm_bits))[4]
 #define v_mask_eq(a, b) ((vector unsigned short) \
   vec_vbpermq((vector unsigned char) vec_cmpeq((a), (b)), perm_bits))[4]
 
@@ -137,26 +135,6 @@ const vector unsigned char perm_bits =
 
 #endif
 
-#if 0
-void dprofile_dump16(WORD * dprofile)
-{
-  printf("\ndprofile:\n");
-  for(int i=0; i<32; i++)
-    {
-      printf("%c: ", sym_nt[i]);
-      for(int k=0; k<CDEPTH; k++)
-        {
-          printf("[");
-          for(int j=0; j<CHANNELS; j++)
-            printf(" %3d", (short) dprofile[CHANNELS*CDEPTH*i + CHANNELS*k + j]);
-          printf("]");
-        }
-      printf("\n");
-    }
-  exit(1);
-}
-#endif
-
 void align_cells_regular_16(VECTORTYPE * Sm,
                             VECTORTYPE * hep,
                             VECTORTYPE ** qp,
@@ -181,32 +159,92 @@ void align_cells_masked_16(VECTORTYPE * Sm,
                            VECTORTYPE * MR,
                            VECTORTYPE * MQ0);
 
-uint64_t backtrack_16(char * qseq,
-                      char * dseq,
-                      uint64_t qlen,
-                      uint64_t dlen,
-                      uint64_t * dirbuffer,
-                      uint64_t offset,
-                      uint64_t dirbuffersize,
-                      uint64_t channel,
-                      uint64_t * alignmentlengthp);
+#ifdef __x86_64__
+
+/* functions defined in sse41.cc */
+
+void align_cells_regular_16_sse41(VECTORTYPE * Sm,
+                                  VECTORTYPE * hep,
+                                  VECTORTYPE ** qp,
+                                  VECTORTYPE * Qm,
+                                  VECTORTYPE * Rm,
+                                  uint64_t ql,
+                                  VECTORTYPE * F0,
+                                  uint64_t * dir_long,
+                                  VECTORTYPE * H0);
+
+void align_cells_masked_16_sse41(VECTORTYPE * Sm,
+                                 VECTORTYPE * hep,
+                                 VECTORTYPE ** qp,
+                                 VECTORTYPE * Qm,
+                                 VECTORTYPE * Rm,
+                                 uint64_t ql,
+                                 VECTORTYPE * F0,
+                                 uint64_t * dir_long,
+                                 VECTORTYPE * H0,
+                                 VECTORTYPE * Mm,
+                                 VECTORTYPE * MQ,
+                                 VECTORTYPE * MR,
+                                 VECTORTYPE * MQ0);
+
+#endif
+
+auto backtrack_16(char * qseq,
+                  char * dseq,
+                  uint64_t qlen,
+                  uint64_t dlen,
+                  uint64_t * dirbuffer,
+                  uint64_t offset,
+                  uint64_t dirbuffersize,
+                  uint64_t channel,
+                  uint64_t * alignmentlengthp) -> uint64_t;
 
 inline void dprofile_fill16(WORD * dprofile_word,
                             WORD * score_matrix_word,
                             BYTE * dseq)
 {
-  VECTORTYPE reg0,  reg1,  reg2,  reg3,  reg4,  reg5,  reg6,  reg7;
-  VECTORTYPE reg8,  reg9,  reg10, reg11, reg12, reg13, reg14, reg15;
-  VECTORTYPE reg16, reg17, reg18, reg19, reg20, reg21, reg22, reg23;
-  VECTORTYPE reg24, reg25, reg26, reg27, reg28, reg29, reg30, reg31;
+  constexpr unsigned int multiplier {5};
+  VECTORTYPE reg0;
+  VECTORTYPE reg1;
+  VECTORTYPE reg2;
+  VECTORTYPE reg3;
+  VECTORTYPE reg4;
+  VECTORTYPE reg5;
+  VECTORTYPE reg6;
+  VECTORTYPE reg7;
+  VECTORTYPE reg8;
+  VECTORTYPE reg9;
+  VECTORTYPE reg10;
+  VECTORTYPE reg11;
+  VECTORTYPE reg12;
+  VECTORTYPE reg13;
+  VECTORTYPE reg14;
+  VECTORTYPE reg15;
+  VECTORTYPE reg16;
+  VECTORTYPE reg17;
+  VECTORTYPE reg18;
+  VECTORTYPE reg19;
+  VECTORTYPE reg20;
+  VECTORTYPE reg21;
+  VECTORTYPE reg22;
+  VECTORTYPE reg23;
+  VECTORTYPE reg24;
+  VECTORTYPE reg25;
+  VECTORTYPE reg26;
+  VECTORTYPE reg27;
+  VECTORTYPE reg28;
+  VECTORTYPE reg29;
+  VECTORTYPE reg30;
+  VECTORTYPE reg31;
 
-  for (unsigned int j=0; j<CDEPTH; j++)
+  for(auto j = 0U; j < cdepth; j++)
     {
-      unsigned int d[CHANNELS];
-      for(unsigned int z=0; z<CHANNELS; z++)
-        d[z] = (static_cast<unsigned int>(dseq[j*CHANNELS+z])) << 5;
+      unsigned int d[channels];
+      for(auto z = 0U; z < channels; z++) {
+        d[z] = (static_cast<unsigned int>(dseq[j * channels + z])) << multiplier;
+      }
 
-      for(int i=0; i<8; i += 8)
+      for(auto i = 0U; i < channels; i += channels)
         {
           reg0  = v_load(score_matrix_word + d[0] + i);
           reg1  = v_load(score_matrix_word + d[1] + i);
@@ -244,14 +282,14 @@ inline void dprofile_fill16(WORD * dprofile_word,
           reg30 = v_merge_lo_64(reg21, reg23);
           reg31 = v_merge_hi_64(reg21, reg23);
 
-          v_store(dprofile_word + CDEPTH*CHANNELS*(i+0) + CHANNELS*j, reg24);
-          v_store(dprofile_word + CDEPTH*CHANNELS*(i+1) + CHANNELS*j, reg25);
-          v_store(dprofile_word + CDEPTH*CHANNELS*(i+2) + CHANNELS*j, reg26);
-          v_store(dprofile_word + CDEPTH*CHANNELS*(i+3) + CHANNELS*j, reg27);
-          v_store(dprofile_word + CDEPTH*CHANNELS*(i+4) + CHANNELS*j, reg28);
-          v_store(dprofile_word + CDEPTH*CHANNELS*(i+5) + CHANNELS*j, reg29);
-          v_store(dprofile_word + CDEPTH*CHANNELS*(i+6) + CHANNELS*j, reg30);
-          v_store(dprofile_word + CDEPTH*CHANNELS*(i+7) + CHANNELS*j, reg31);
+          v_store(dprofile_word + cdepth * channels * (i + 0) + channels * j, reg24);
+          v_store(dprofile_word + cdepth * channels * (i + 1) + channels * j, reg25);
+          v_store(dprofile_word + cdepth * channels * (i + 2) + channels * j, reg26);
+          v_store(dprofile_word + cdepth * channels * (i + 3) + channels * j, reg27);
+          v_store(dprofile_word + cdepth * channels * (i + 4) + channels * j, reg28);
+          v_store(dprofile_word + cdepth * channels * (i + 5) + channels * j, reg29);
+          v_store(dprofile_word + cdepth * channels * (i + 6) + channels * j, reg30);
+          v_store(dprofile_word + cdepth * channels * (i + 7) + channels * j, reg31);
         }
     }
 #if 0
@@ -268,19 +306,22 @@ inline void onestep_16(VECTORTYPE & H,
                        VECTORTYPE QR,
                        VECTORTYPE R)
 {
+  VECTORTYPE W;
+
   H = v_add(H, V);
-  *(DIR+0) = v_mask_gt(H, F);
+  W = H;
   H = v_min(H, F);
+  *(DIR+0) = v_mask_eq(W, H);
   H = v_min(H, E);
   *(DIR+1) = v_mask_eq(H, E);
   N = H;
   H = v_add(H, QR);
   F = v_add(F, R);
   E = v_add(E, R);
-  *(DIR+2) = v_mask_gt(H, F);
-  *(DIR+3) = v_mask_gt(H, E);
   F = v_min(H, F);
+  *(DIR+2) = v_mask_eq(H, F);
   E = v_min(H, E);
+  *(DIR+3) = v_mask_eq(H, E);
 }
 
 void align_cells_regular_16(VECTORTYPE * Sm,
@@ -293,11 +334,30 @@ void align_cells_regular_16(VECTORTYPE * Sm,
                             uint64_t * dir_long,
                             VECTORTYPE * H0)
 {
-  VECTORTYPE Q, R, E;
-  VECTORTYPE h0, h1, h2, h3, h4, h5, h6, h7, h8;
-  VECTORTYPE f0, f1, f2, f3;
+  constexpr unsigned int step {16};
+  constexpr unsigned int offset0 {0};
+  constexpr unsigned int offset1 {offset0 + 4};
+  constexpr unsigned int offset2 {offset1 + 4};
+  constexpr unsigned int offset3 {offset2 + 4};
 
-  unsigned short * dir = reinterpret_cast<unsigned short *>(dir_long);
+  VECTORTYPE Q;
+  VECTORTYPE R;
+  VECTORTYPE E;
+  VECTORTYPE h0;
+  VECTORTYPE h1;
+  VECTORTYPE h2;
+  VECTORTYPE h3;
+  VECTORTYPE h4;
+  VECTORTYPE h5;
+  VECTORTYPE h6;
+  VECTORTYPE h7;
+  VECTORTYPE h8;
+  VECTORTYPE f0;
+  VECTORTYPE f1;
+  VECTORTYPE f2;
+  VECTORTYPE f3;
+
+  auto * dir = reinterpret_cast<unsigned short *>(dir_long);
 
   Q = *Qm;
   R = *Rm;
@@ -311,25 +371,25 @@ void align_cells_regular_16(VECTORTYPE * Sm,
   h1 = v_sub(f0, Q);
   h2 = v_add(h1, R);
   h3 = v_add(h2, R);
-  h4 = v_zero;
+
   h5 = v_zero;
   h6 = v_zero;
   h7 = v_zero;
   h8 = v_zero;
 
-  for(uint64_t i = 0; i < ql; i++)
+  for(auto i = 0ULL; i < ql; i++)
     {
-      VECTORTYPE * x;
+      VECTORTYPE *x {nullptr};
 
       x = qp[i + 0];
-      h4 = hep[2*i + 0];
-      E  = hep[2*i + 1];
-      onestep_16(h0, h5, f0, x[0], dir + 16*i +  0, E, Q, R);
-      onestep_16(h1, h6, f1, x[1], dir + 16*i +  4, E, Q, R);
-      onestep_16(h2, h7, f2, x[2], dir + 16*i +  8, E, Q, R);
-      onestep_16(h3, h8, f3, x[3], dir + 16*i + 12, E, Q, R);
-      hep[2*i + 0] = h8;
-      hep[2*i + 1] = E;
+      h4 = hep[2 * i + 0];
+      E  = hep[2 * i + 1];
+      onestep_16(h0, h5, f0, x[0], dir + step * i + offset0, E, Q, R);
+      onestep_16(h1, h6, f1, x[1], dir + step * i + offset1, E, Q, R);
+      onestep_16(h2, h7, f2, x[2], dir + step * i + offset2, E, Q, R);
+      onestep_16(h3, h8, f3, x[3], dir + step * i + offset3, E, Q, R);
+      hep[2 * i + 0] = h8;
+      hep[2 * i + 1] = E;
       h0 = h4;
       h1 = h5;
       h2 = h6;
@@ -356,11 +416,30 @@ void align_cells_masked_16(VECTORTYPE * Sm,
                            VECTORTYPE * MR,
                            VECTORTYPE * MQ0)
 {
-  VECTORTYPE Q, R, E;
-  VECTORTYPE h0, h1, h2, h3, h4, h5, h6, h7, h8;
-  VECTORTYPE f0, f1, f2, f3;
+  constexpr unsigned int step {16};
+  constexpr unsigned int offset0 {0};
+  constexpr unsigned int offset1 {offset0 + 4};
+  constexpr unsigned int offset2 {offset1 + 4};
+  constexpr unsigned int offset3 {offset2 + 4};
 
-  unsigned short * dir = reinterpret_cast<unsigned short *>(dir_long);
+  VECTORTYPE Q;
+  VECTORTYPE R;
+  VECTORTYPE E;
+  VECTORTYPE h0;
+  VECTORTYPE h1;
+  VECTORTYPE h2;
+  VECTORTYPE h3;
+  VECTORTYPE h4;
+  VECTORTYPE h5;
+  VECTORTYPE h6;
+  VECTORTYPE h7;
+  VECTORTYPE h8;
+  VECTORTYPE f0;
+  VECTORTYPE f1;
+  VECTORTYPE f2;
+  VECTORTYPE f3;
+
+  auto * dir = reinterpret_cast<unsigned short *>(dir_long);
 
   Q = *Qm;
   R = *Rm;
@@ -374,18 +453,18 @@ void align_cells_masked_16(VECTORTYPE * Sm,
   h1 = v_sub(f0, Q);
   h2 = v_add(h1, R);
   h3 = v_add(h2, R);
-  h4 = v_zero;
+
   h5 = v_zero;
   h6 = v_zero;
   h7 = v_zero;
   h8 = v_zero;
 
-  for(uint64_t i = 0; i < ql; i++)
+  for(auto i = 0ULL; i < ql; i++)
     {
-      VECTORTYPE * x;
+      VECTORTYPE *x {nullptr};
 
-      h4 = hep[2*i + 0];
-      E  = hep[2*i + 1];
+      h4 = hep[2 * i + 0];
+      E  = hep[2 * i + 1];
       x = qp[i + 0];
 
       /* mask h4 and E */
@@ -400,12 +479,12 @@ void align_cells_masked_16(VECTORTYPE * Sm,
       /* update MQ */
       *MQ = v_add(*MQ,  *MR);
 
-      onestep_16(h0, h5, f0, x[0], dir + 16*i +  0, E, Q, R);
-      onestep_16(h1, h6, f1, x[1], dir + 16*i +  4, E, Q, R);
-      onestep_16(h2, h7, f2, x[2], dir + 16*i +  8, E, Q, R);
-      onestep_16(h3, h8, f3, x[3], dir + 16*i + 12, E, Q, R);
-      hep[2*i + 0] = h8;
-      hep[2*i + 1] = E;
+      onestep_16(h0, h5, f0, x[0], dir + step * i + offset0, E, Q, R);
+      onestep_16(h1, h6, f1, x[1], dir + step * i + offset1, E, Q, R);
+      onestep_16(h2, h7, f2, x[2], dir + step * i + offset2, E, Q, R);
+      onestep_16(h3, h8, f3, x[3], dir + step * i + offset3, E, Q, R);
+      hep[2 * i + 0] = h8;
+      hep[2 * i + 1] = E;
 
       h0 = h4;
       h1 = h5;
@@ -419,7 +498,7 @@ void align_cells_masked_16(VECTORTYPE * Sm,
   Sm[3] = h8;
 }
 
-uint64_t backtrack_16(char * qseq,
+auto backtrack_16(char * qseq,
                       char * dseq,
                       uint64_t qlen,
                       uint64_t dlen,
@@ -427,20 +506,24 @@ uint64_t backtrack_16(char * qseq,
                       uint64_t offset,
                       uint64_t dirbuffersize,
                       uint64_t channel,
-                      uint64_t * alignmentlengthp)
+                      uint64_t * alignmentlengthp) -> uint64_t
 {
-  uint64_t maskup      = 3ULL << (2*channel+ 0);
-  uint64_t maskleft    = 3ULL << (2*channel+16);
-  uint64_t maskextup   = 3ULL << (2*channel+32);
-  uint64_t maskextleft = 3ULL << (2*channel+48);
+  constexpr unsigned int offset0 {0};
+  constexpr unsigned int offset1 {offset0 + 16};
+  constexpr unsigned int offset2 {offset1 + 16};
+  constexpr unsigned int offset3 {offset2 + 16};
+  uint64_t maskup      = 3ULL << (2 * channel + offset0);
+  uint64_t maskleft    = 3ULL << (2 * channel + offset1);
+  uint64_t maskextup   = 3ULL << (2 * channel + offset2);
+  uint64_t maskextleft = 3ULL << (2 * channel + offset3);
 
 #if 0
 
   printf("Dumping backtracking array\n");
 
-  for(uint64_t i=0; i<qlen; i++)
+  for(auto i = 0ULL; i < qlen; i++)
     {
-      for(uint64_t j=0; j<dlen; j++)
+      for(auto j = 0ULL; j < dlen; j++)
         {
           uint64_t d = dirbuffer[(offset + longestdbsequence * 4 * (j / 4)
                                   + 4 * i + (j & 3)) % dirbuffersize];
@@ -448,7 +531,7 @@ uint64_t backtrack_16(char * qseq,
             {
               printf("<");
             }
-          else if (d & maskup)
+          else if (!(d & maskup))
             {
               printf("^");
             }
@@ -462,20 +545,20 @@ uint64_t backtrack_16(char * qseq,
 
   printf("Dumping gap extension array\n");
 
-  for(uint64_t i=0; i<qlen; i++)
+  for(auto i = 0ULL; i < qlen; i++)
     {
-      for(uint64_t j=0; j<dlen; j++)
+      for(auto j = 0ULL; j < dlen; j++)
         {
           uint64_t d = dirbuffer[(offset + longestdbsequence * 4 *(j / 4)
                                   + 4 * i + (j & 3)) % dirbuffersize];
-          if (d & maskextup)
+          if (!(d & maskextup))
             {
-              if (d & maskextleft)
+              if (!(d & maskextleft))
                 printf("+");
               else
                 printf("^");
             }
-          else if (d & maskextleft)
+          else if (!(d & maskextleft))
             {
               printf("<");
             }
@@ -491,9 +574,9 @@ uint64_t backtrack_16(char * qseq,
 
   int64_t i = static_cast<int64_t>(qlen) - 1;
   int64_t j = static_cast<int64_t>(dlen) - 1;
-  uint64_t aligned = 0;
-  uint64_t matches = 0;
-  char op = 0;
+  uint64_t aligned {0};
+  uint64_t matches {0};
+  char op {0};
 
 #undef SHOWALIGNMENT
 #ifdef SHOWALIGNMENT
@@ -510,20 +593,20 @@ uint64_t backtrack_16(char * qseq,
                      + static_cast<uint64_t>(4 * i + (j & 3)))
                     % dirbuffersize];
 
-      if ((op == 'I') && (d & maskextleft))
+      if ((op == 'I') && ((d & maskextleft) == 0U))
         {
           j--;
         }
-      else if ((op == 'D') && (d & maskextup))
+      else if ((op == 'D') && ((d & maskextup) == 0U))
         {
           i--;
         }
-      else if (d & maskleft)
+      else if ((d & maskleft) != 0U)
         {
           j--;
           op = 'I';
         }
-      else if (d & maskup)
+      else if ((d & maskup) == 0U)
         {
           i--;
           op = 'D';
@@ -531,8 +614,9 @@ uint64_t backtrack_16(char * qseq,
       else
         {
           if (nt_extract(qseq, static_cast<uint64_t>(i)) ==
-              nt_extract(dseq, static_cast<uint64_t>(j)))
+              nt_extract(dseq, static_cast<uint64_t>(j))) {
             matches++;
+          }
           i--;
           j--;
           op = 'M';
@@ -584,15 +668,23 @@ void search16(WORD * * q_start,
               uint64_t dirbuffersize,
               uint64_t * dirbuffer)
 {
-  VECTORTYPE Q, R, T, M, T0, MQ, MR, MQ0;
-  VECTORTYPE *hep, **qp;
+  VECTORTYPE Q;
+  VECTORTYPE R;
+  VECTORTYPE T;
+  VECTORTYPE M;
+  VECTORTYPE T0;
+  VECTORTYPE MQ;
+  VECTORTYPE MR;
+  VECTORTYPE MQ0;
+  VECTORTYPE *hep {nullptr};
+  VECTORTYPE **qp {nullptr};
 
-  uint64_t d_pos[CHANNELS];
-  uint64_t d_offset[CHANNELS];
-  char * d_address[CHANNELS];
-  uint64_t d_length[CHANNELS];
+  uint64_t d_pos[channels];
+  uint64_t d_offset[channels];
+  char * d_address[channels];
+  uint64_t d_length[channels];
 
-  VECTORTYPE dseqalloc[CDEPTH];
+  VECTORTYPE dseqalloc[cdepth];
 
   VECTORTYPE H0;
   VECTORTYPE F0;
@@ -600,9 +692,9 @@ void search16(WORD * * q_start,
 
   BYTE * dseq = reinterpret_cast<BYTE*>(& dseqalloc);
 
-  int64_t seq_id[CHANNELS];
-  uint64_t next_id = 0;
-  uint64_t done;
+  int64_t seq_id[channels];
+  uint64_t next_id {0};
+  uint64_t done {0};
 
 #ifdef __aarch64__
   const VECTORTYPE T0_init = { -1, 0, 0, 0, 0, 0, 0, 0 };
@@ -622,7 +714,7 @@ void search16(WORD * * q_start,
   hep = CAST_VECTOR_p(hearray);
   qp = reinterpret_cast<VECTORTYPE**>(q_start);
 
-  for (int c=0; c<CHANNELS; c++)
+  for(auto c = 0U; c < channels; c++)
     {
       d_address[c] = nullptr;
       d_pos[c] = 0;
@@ -633,65 +725,84 @@ void search16(WORD * * q_start,
   F0 = v_zero;
   H0 = v_zero;
 
-  int easy = 0;
+  bool easy {false};
 
   uint64_t * dir = dirbuffer;
 
-  while(1)
+  while(true)
     {
 
       if (easy)
         {
           // fill all channels
 
-          for(int c=0; c<CHANNELS; c++)
+          for(auto c = 0U; c < channels; c++)
             {
-              for(int j=0; j<CDEPTH; j++)
+              for(auto j = 0U; j < cdepth; j++)
                 {
-                  if (d_pos[c] < d_length[c])
-                    dseq[CHANNELS*j+c]
+                  if (d_pos[c] < d_length[c]) {
+                    dseq[channels * j + c]
                       = 1 + nt_extract(d_address[c], d_pos[c]++);
-                  else
-                    dseq[CHANNELS*j+c] = 0;
+                  }
+                  else {
+                    dseq[channels * j + c] = 0;
+                  }
                 }
-              if (d_pos[c] == d_length[c])
-                easy = 0;
+              if (d_pos[c] == d_length[c]) {
+                easy = false;
+              }
             }
 
 #ifdef __x86_64__
-          if (ssse3_present)
-            dprofile_shuffle16(dprofile, score_matrix, dseq);
+          if (ssse3_present != 0)
+            {
+              dprofile_shuffle16(dprofile, score_matrix, dseq);
+            }
           else
 #endif
-            dprofile_fill16(dprofile, score_matrix, dseq);
+            {
+              dprofile_fill16(dprofile, score_matrix, dseq);
+            }
 
-          align_cells_regular_16(S, hep, qp, &Q, &R, qlen, &F0, dir, &H0);
+#ifdef __x86_64__
+          if (sse41_present)
+            {
+              align_cells_regular_16_sse41(S, hep, qp, &Q, &R, qlen, &F0, dir, &H0);
+            }
+          else
+#endif
+            {
+              align_cells_regular_16(S, hep, qp, &Q, &R, qlen, &F0, dir, &H0);
+            }
         }
       else
         {
           // One or more sequences ended in the previous block
           // We have to switch over to a new sequence
 
-          easy = 1;
+          easy = true;
 
           M = v_zero;
           T = T0;
-          for (unsigned int c = 0; c < CHANNELS; c++)
+          for(auto c = 0U; c < channels; c++)
             {
               if (d_pos[c] < d_length[c])
                 {
                   // this channel has more sequence
 
-                  for(unsigned int j = 0; j < CDEPTH; j++)
+                  for(auto j = 0U; j < cdepth; j++)
                     {
-                      if (d_pos[c] < d_length[c])
-                        dseq[CHANNELS * j + c]
+                      if (d_pos[c] < d_length[c]) {
+                        dseq[channels * j + c]
                           = 1 + nt_extract(d_address[c], d_pos[c]++);
-                      else
-                        dseq[CHANNELS*j+c] = 0;
+                      }
+                      else {
+                        dseq[channels*j+c] = 0;
+                      }
                     }
-                  if (d_pos[c] == d_length[c])
-                    easy = 0;
+                  if (d_pos[c] == d_length[c]) {
+                    easy = false;
+                  }
                 }
               else
                 {
@@ -710,12 +821,12 @@ void search16(WORD * * q_start,
                       uint64_t dbseqlen = d_length[c];
                       uint64_t z = (dbseqlen+3) % 4;
                       uint64_t score
-                        = (reinterpret_cast<WORD*>(S))[z * CHANNELS + c];
+                        = (reinterpret_cast<WORD*>(S))[z * channels + c];
                       scores[cand_id] = score;
 
-                      uint64_t diff;
+                      uint64_t diff {0};
 
-                      if (score < 65535)
+                      if (score < UINT16_MAX)
                         {
                           uint64_t offset = d_offset[c];
                           diff = backtrack_16(query.seq, dbseq, qlen, dbseqlen,
@@ -726,10 +837,7 @@ void search16(WORD * * q_start,
                         }
                       else
                         {
-                          diff = static_cast<uint64_t>
-                            (MIN((65535 / penalty_mismatch),
-                                 (65535 - penalty_gapopen)
-                                 / penalty_gapextend));
+                          diff = UINT16_MAX;
                         }
 
                       diffs[cand_id] = diff;
@@ -742,8 +850,8 @@ void search16(WORD * * q_start,
                       // get next sequence
                       seq_id[c] = static_cast<int64_t>(next_id);
                       uint64_t seqno = seqnos[next_id];
-                      char* address;
-                      unsigned int length;
+                      char * address {nullptr};
+                      unsigned int length {0};
 
                       db_getsequenceandlength(seqno, & address, & length);
 
@@ -758,15 +866,18 @@ void search16(WORD * * q_start,
                       (reinterpret_cast<WORD*>(&F0))[c] = 2 * gap_open_penalty + 2 * gap_extend_penalty;
 
                       // fill channel
-                      for(unsigned int j = 0; j < CDEPTH; j++)
+                      for(auto j = 0U; j < cdepth; j++)
                         {
-                          if (d_pos[c] < d_length[c])
-                            dseq[CHANNELS*j+c] = 1 + nt_extract(d_address[c], d_pos[c]++);
-                          else
-                            dseq[CHANNELS*j+c] = 0;
+                          if (d_pos[c] < d_length[c]) {
+                            dseq[channels * j + c] = 1 + nt_extract(d_address[c], d_pos[c]++);
+                          }
+                          else {
+                            dseq[channels * j + c] = 0;
+                          }
                         }
-                      if (d_pos[c] == d_length[c])
-                        easy = 0;
+                      if (d_pos[c] == d_length[c]) {
+                        easy = false;
+                      }
                     }
                   else
                     {
@@ -775,29 +886,44 @@ void search16(WORD * * q_start,
                       d_address[c] = nullptr;
                       d_pos[c] = 0;
                       d_length[c] = 0;
-                      for (unsigned int j=0; j<CDEPTH; j++)
-                        dseq[CHANNELS*j+c] = 0;
+                      for(auto j = 0U; j < cdepth; j++) {
+                        dseq[channels * j + c] = 0;
+                      }
                     }
                 }
 
               T = v_shift_left(T);
             }
 
-          if (done == sequences)
+          if (done == sequences) {
             break;
+          }
 
 #ifdef __x86_64__
-          if (ssse3_present)
-            dprofile_shuffle16(dprofile, score_matrix, dseq);
+          if (ssse3_present != 0)
+            {
+              dprofile_shuffle16(dprofile, score_matrix, dseq);
+            }
           else
 #endif
-            dprofile_fill16(dprofile, score_matrix, dseq);
+            {
+              dprofile_fill16(dprofile, score_matrix, dseq);
+            }
 
           MQ = v_and(M, Q);
           MR = v_and(M, R);
           MQ0 = MQ;
 
-          align_cells_masked_16(S, hep, qp, &Q, &R, qlen, &F0, dir, &H0, &M, &MQ, &MR, &MQ0);
+#ifdef __x86_64__
+          if (sse41_present)
+            {
+              align_cells_masked_16_sse41(S, hep, qp, &Q, &R, qlen, &F0, dir, &H0, &M, &MQ, &MR, &MQ0);
+            }
+          else
+#endif
+            {
+              align_cells_masked_16(S, hep, qp, &Q, &R, qlen, &F0, dir, &H0, &M, &MQ, &MR, &MQ0);
+            }
         }
 
       F0 = v_add(F0, R);
@@ -806,8 +932,9 @@ void search16(WORD * * q_start,
       H0 = v_sub(F0, Q);
       F0 = v_add(F0, R);
 
-      dir += 4*longestdbsequence;
-      if (dir >= dirbuffer + dirbuffersize)
+      dir += 4 * longestdbsequence;
+      if (dir >= dirbuffer + dirbuffersize) {
         dir -= dirbuffersize;
+      }
     }
 }
