@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2019 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -23,16 +23,16 @@
 
 #include "swarm.h"
 
-#define CHANNELS 16
-#define CDEPTH 4
+constexpr unsigned int channels {16};
+constexpr unsigned int cdepth {4};
 
 /* uses 16 unsigned 8-bit values */
 
 #ifdef __aarch64__
 
-typedef int8x16_t VECTORTYPE;
+using VECTORTYPE = int8x16_t;
 
-#define CAST_VECTOR_p(x) reinterpret_cast<VECTORTYPE *>(x)
+#define CAST_VECTOR_p(x) (reinterpret_cast<VECTORTYPE *>(x))
 
 const uint8x16_t neon_mask =
   { 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80,
@@ -68,9 +68,9 @@ const uint16x8_t neon_shift = { 0, 0, 0, 0, 8, 8, 8, 8 };
 
 #elif defined __x86_64__
 
-typedef __m128i VECTORTYPE;
+using VECTORTYPE = __m128i;
 
-#define CAST_VECTOR_p(x) reinterpret_cast<VECTORTYPE *>(x)
+#define CAST_VECTOR_p(x) (reinterpret_cast<VECTORTYPE *>(x))
 
 #define v_load_64(a) _mm_loadl_epi64(CAST_VECTOR_p(a))
 #define v_store(a, b) _mm_store_si128(CAST_VECTOR_p(a), (b))
@@ -93,7 +93,7 @@ typedef __m128i VECTORTYPE;
 
 #elif defined __PPC__
 
-typedef vector unsigned char VECTORTYPE;
+using VECTORTYPE = vector unsigned char;
 
 #define CAST_VECTOR_p(x) reinterpret_cast<VECTORTYPE *>(x)
 
@@ -168,38 +168,34 @@ void align_cells_masked_8(VECTORTYPE * Sm,
                           VECTORTYPE * MR,
                           VECTORTYPE * MQ0);
 
-#if 0
-void dprofile_dump8(BYTE * dprofile)
-{
-  printf("\ndprofile:\n");
-  for(int k=0; k<4; k++)
-    {
-      printf("k=%d 0 1 2 3 4 5 6 7 8 9 a b c d e f\n", k);
-      for(int i=0; i<16; i++)
-        {
-          printf("%c: ", sym_nt[i]);
-          for(int j = 0; j < 16; j++)
-            printf("%2d", (char) dprofile[i*64+16*k + j]);
-          printf("\n");
-        }
-    }
-  printf("\n");
-  exit(1);
-}
-#endif
-
 inline void dprofile_fill8(BYTE * dprofile,
                            BYTE * score_matrix,
                            BYTE * dseq)
 {
-  VECTORTYPE reg0,  reg1, reg2,  reg3,  reg4,  reg5,  reg6,  reg7;
-  VECTORTYPE reg8,  reg9, reg10, reg11, reg12, reg13, reg14, reg15;
+  constexpr unsigned int multiplier {5};
+  VECTORTYPE reg0;
+  VECTORTYPE reg1;
+  VECTORTYPE reg2;
+  VECTORTYPE reg3;
+  VECTORTYPE reg4;
+  VECTORTYPE reg5;
+  VECTORTYPE reg6;
+  VECTORTYPE reg7;
+  VECTORTYPE reg8;
+  VECTORTYPE reg9;
+  VECTORTYPE reg10;
+  VECTORTYPE reg11;
+  VECTORTYPE reg12;
+  VECTORTYPE reg13;
+  VECTORTYPE reg14;
+  VECTORTYPE reg15;
 
-  for(unsigned int j=0; j<CDEPTH; j++)
+  for(auto j = 0U; j < cdepth; j++)
     {
-      unsigned int d[CHANNELS];
-      for(unsigned int i = 0; i < CHANNELS; i++)
-        d[i] = (static_cast<unsigned int>(dseq[j * CHANNELS + i])) << 5;
+      unsigned int d[channels];
+      for(auto i = 0U; i < channels; i++) {
+        d[i] = (static_cast<unsigned int>(dseq[j * channels + i])) << multiplier;
+      }
 
       reg0  = v_load_64(score_matrix + d[ 0]);
       reg2  = v_load_64(score_matrix + d[ 2]);
@@ -258,14 +254,14 @@ inline void dprofile_fill8(BYTE * dprofile,
       reg6  = v_merge_lo_64(reg6, reg14);
       reg15 = v_merge_hi_64(reg15, reg14);
 
-      v_store(dprofile+16*j+  0, reg0);
-      v_store(dprofile+16*j+ 64, reg3);
-      v_store(dprofile+16*j+128, reg2);
-      v_store(dprofile+16*j+192, reg7);
-      v_store(dprofile+16*j+256, reg1);
-      v_store(dprofile+16*j+320, reg11);
-      v_store(dprofile+16*j+384, reg6);
-      v_store(dprofile+16*j+448, reg15);
+      v_store(dprofile + 16 * j +   0, reg0);
+      v_store(dprofile + 16 * j +  64, reg3);
+      v_store(dprofile + 16 * j + 128, reg2);
+      v_store(dprofile + 16 * j + 192, reg7);
+      v_store(dprofile + 16 * j + 256, reg1);
+      v_store(dprofile + 16 * j + 320, reg11);
+      v_store(dprofile + 16 * j + 384, reg6);
+      v_store(dprofile + 16 * j + 448, reg15);
 
 
       // loads not aligned on 16 byte boundary, cannot load and unpack in one instr.
@@ -335,14 +331,14 @@ inline void dprofile_fill8(BYTE * dprofile,
       reg6  = v_merge_lo_64(reg6, reg14);
       reg15 = v_merge_hi_64(reg15, reg14);
 
-      v_store(dprofile+16*j+512+  0, reg0);
-      v_store(dprofile+16*j+512+ 64, reg3);
-      v_store(dprofile+16*j+512+128, reg2);
-      v_store(dprofile+16*j+512+192, reg7);
-      v_store(dprofile+16*j+512+256, reg1);
-      v_store(dprofile+16*j+512+320, reg11);
-      v_store(dprofile+16*j+512+384, reg6);
-      v_store(dprofile+16*j+512+448, reg15);
+      v_store(dprofile + 16 * j + 512 +   0, reg0);
+      v_store(dprofile + 16 * j + 512 +  64, reg3);
+      v_store(dprofile + 16 * j + 512 + 128, reg2);
+      v_store(dprofile + 16 * j + 512 + 192, reg7);
+      v_store(dprofile + 16 * j + 512 + 256, reg1);
+      v_store(dprofile + 16 * j + 512 + 320, reg11);
+      v_store(dprofile + 16 * j + 512 + 384, reg6);
+      v_store(dprofile + 16 * j + 512 + 448, reg15);
 
 
       reg0  = v_load_64(score_matrix + 16 + d[0 ]);
@@ -402,14 +398,14 @@ inline void dprofile_fill8(BYTE * dprofile,
       reg6  = v_merge_lo_64(reg6, reg14);
       reg15 = v_merge_hi_64(reg15, reg14);
 
-      v_store(dprofile+16*j+1024+  0, reg0);
-      v_store(dprofile+16*j+1024+ 64, reg3);
-      v_store(dprofile+16*j+1024+128, reg2);
-      v_store(dprofile+16*j+1024+192, reg7);
-      v_store(dprofile+16*j+1024+256, reg1);
-      v_store(dprofile+16*j+1024+320, reg11);
-      v_store(dprofile+16*j+1024+384, reg6);
-      v_store(dprofile+16*j+1024+448, reg15);
+      v_store(dprofile + 16 * j + 1024 +   0, reg0);
+      v_store(dprofile + 16 * j + 1024 +  64, reg3);
+      v_store(dprofile + 16 * j + 1024 + 128, reg2);
+      v_store(dprofile + 16 * j + 1024 + 192, reg7);
+      v_store(dprofile + 16 * j + 1024 + 256, reg1);
+      v_store(dprofile + 16 * j + 1024 + 320, reg11);
+      v_store(dprofile + 16 * j + 1024 + 384, reg6);
+      v_store(dprofile + 16 * j + 1024 + 448, reg15);
 
 
       // loads not aligned on 16 byte boundary, cannot load and unpack in one instr.
@@ -479,14 +475,14 @@ inline void dprofile_fill8(BYTE * dprofile,
       reg6  = v_merge_lo_64(reg6, reg14);
       reg15 = v_merge_hi_64(reg15, reg14);
 
-      v_store(dprofile+16*j+1536+  0, reg0);
-      v_store(dprofile+16*j+1536+ 64, reg3);
-      v_store(dprofile+16*j+1536+128, reg2);
-      v_store(dprofile+16*j+1536+192, reg7);
-      v_store(dprofile+16*j+1536+256, reg1);
-      v_store(dprofile+16*j+1536+320, reg11);
-      v_store(dprofile+16*j+1536+384, reg6);
-      v_store(dprofile+16*j+1536+448, reg15);
+      v_store(dprofile + 16 * j + 1536 +   0, reg0);
+      v_store(dprofile + 16 * j + 1536 +  64, reg3);
+      v_store(dprofile + 16 * j + 1536 + 128, reg2);
+      v_store(dprofile + 16 * j + 1536 + 192, reg7);
+      v_store(dprofile + 16 * j + 1536 + 256, reg1);
+      v_store(dprofile + 16 * j + 1536 + 320, reg11);
+      v_store(dprofile + 16 * j + 1536 + 384, reg6);
+      v_store(dprofile + 16 * j + 1536 + 448, reg15);
     }
 #if 0
   dprofile_dump8(dprofile);
@@ -530,11 +526,24 @@ void align_cells_regular_8(VECTORTYPE * Sm,
                            uint64_t * dir_long,
                            VECTORTYPE * H0)
 {
-  VECTORTYPE Q, R, E;
-  VECTORTYPE h0, h1, h2, h3, h4, h5, h6, h7, h8;
-  VECTORTYPE f0, f1, f2, f3;
+  VECTORTYPE Q;
+  VECTORTYPE R;
+  VECTORTYPE E;
+  VECTORTYPE h0;
+  VECTORTYPE h1;
+  VECTORTYPE h2;
+  VECTORTYPE h3;
+  VECTORTYPE h4;
+  VECTORTYPE h5;
+  VECTORTYPE h6;
+  VECTORTYPE h7;
+  VECTORTYPE h8;
+  VECTORTYPE f0;
+  VECTORTYPE f1;
+  VECTORTYPE f2;
+  VECTORTYPE f3;
 
-  unsigned short * dir = reinterpret_cast<unsigned short *>(dir_long);
+  auto * dir = reinterpret_cast<unsigned short *>(dir_long);
 
   Q = *Qm;
   R = *Rm;
@@ -548,25 +557,29 @@ void align_cells_regular_8(VECTORTYPE * Sm,
   h1 = v_sub(f0, Q);
   h2 = v_add(h1, R);
   h3 = v_add(h2, R);
-  h4 = v_zero;
+
   h5 = v_zero;
   h6 = v_zero;
   h7 = v_zero;
   h8 = v_zero;
 
-  for(uint64_t i = 0; i < ql; i++)
+  for(auto i = 0ULL; i < ql; i++)
     {
-      VECTORTYPE * x;
+      VECTORTYPE *x {nullptr};
+      constexpr unsigned int j0 {0};
+      constexpr unsigned int j1 {j0 + 4};
+      constexpr unsigned int j2 {j1 + 4};
+      constexpr unsigned int j3 {j2 + 4};
 
       x = qp[i + 0];
-      h4 = hep[2*i + 0];
-      E  = hep[2*i + 1];
-      onestep_8(h0, h5, f0, x[0], dir + 16*i +  0, E, Q, R);
-      onestep_8(h1, h6, f1, x[1], dir + 16*i +  4, E, Q, R);
-      onestep_8(h2, h7, f2, x[2], dir + 16*i +  8, E, Q, R);
-      onestep_8(h3, h8, f3, x[3], dir + 16*i + 12, E, Q, R);
-      hep[2*i + 0] = h8;
-      hep[2*i + 1] = E;
+      h4 = hep[2 * i + 0];
+      E  = hep[2 * i + 1];
+      onestep_8(h0, h5, f0, x[0], dir + channels * i + j0, E, Q, R);
+      onestep_8(h1, h6, f1, x[1], dir + channels * i + j1, E, Q, R);
+      onestep_8(h2, h7, f2, x[2], dir + channels * i + j2, E, Q, R);
+      onestep_8(h3, h8, f3, x[3], dir + channels * i + j3, E, Q, R);
+      hep[2 * i + 0] = h8;
+      hep[2 * i + 1] = E;
       h0 = h4;
       h1 = h5;
       h2 = h6;
@@ -593,11 +606,24 @@ void align_cells_masked_8(VECTORTYPE * Sm,
                           VECTORTYPE * MR,
                           VECTORTYPE * MQ0)
 {
-  VECTORTYPE Q, R, E;
-  VECTORTYPE h0, h1, h2, h3, h4, h5, h6, h7, h8;
-  VECTORTYPE f0, f1, f2, f3;
+  VECTORTYPE Q;
+  VECTORTYPE R;
+  VECTORTYPE E;
+  VECTORTYPE h0;
+  VECTORTYPE h1;
+  VECTORTYPE h2;
+  VECTORTYPE h3;
+  VECTORTYPE h4;
+  VECTORTYPE h5;
+  VECTORTYPE h6;
+  VECTORTYPE h7;
+  VECTORTYPE h8;
+  VECTORTYPE f0;
+  VECTORTYPE f1;
+  VECTORTYPE f2;
+  VECTORTYPE f3;
 
-  unsigned short * dir = reinterpret_cast<unsigned short *>(dir_long);
+  auto * dir = reinterpret_cast<unsigned short *>(dir_long);
 
   Q = *Qm;
   R = *Rm;
@@ -611,18 +637,23 @@ void align_cells_masked_8(VECTORTYPE * Sm,
   h1 = v_sub(f0, Q);
   h2 = v_add(h1, R);
   h3 = v_add(h2, R);
-  h4 = v_zero;
+
   h5 = v_zero;
   h6 = v_zero;
   h7 = v_zero;
   h8 = v_zero;
 
-  for(uint64_t i = 0; i < ql; i++)
+  for(auto i = 0ULL; i < ql; i++)
     {
-      VECTORTYPE * x;
+      constexpr unsigned int j0 {0};
+      constexpr unsigned int j1 {j0 + 4};
+      constexpr unsigned int j2 {j1 + 4};
+      constexpr unsigned int j3 {j2 + 4};
 
-      h4 = hep[2*i + 0];
-      E  = hep[2*i + 1];
+      VECTORTYPE * x {nullptr};
+
+      h4 = hep[2 * i + 0];
+      E  = hep[2 * i + 1];
       x = qp[i + 0];
 
       /* mask h4 and E */
@@ -637,12 +668,12 @@ void align_cells_masked_8(VECTORTYPE * Sm,
       /* update MQ */
       *MQ = v_add(*MQ,  *MR);
 
-      onestep_8(h0, h5, f0, x[0], dir + 16*i +  0, E, Q, R);
-      onestep_8(h1, h6, f1, x[1], dir + 16*i +  4, E, Q, R);
-      onestep_8(h2, h7, f2, x[2], dir + 16*i +  8, E, Q, R);
-      onestep_8(h3, h8, f3, x[3], dir + 16*i + 12, E, Q, R);
-      hep[2*i + 0] = h8;
-      hep[2*i + 1] = E;
+      onestep_8(h0, h5, f0, x[0], dir + channels * i + j0, E, Q, R);
+      onestep_8(h1, h6, f1, x[1], dir + channels * i + j1, E, Q, R);
+      onestep_8(h2, h7, f2, x[2], dir + channels * i + j2, E, Q, R);
+      onestep_8(h3, h8, f3, x[3], dir + channels * i + j3, E, Q, R);
+      hep[2 * i + 0] = h8;
+      hep[2 * i + 1] = E;
 
       h0 = h4;
       h1 = h5;
@@ -656,7 +687,7 @@ void align_cells_masked_8(VECTORTYPE * Sm,
   Sm[3] = h8;
 }
 
-inline uint64_t backtrack_8(char * qseq,
+inline auto backtrack_8(char * qseq,
                             char * dseq,
                             uint64_t qlen,
                             uint64_t dlen,
@@ -664,20 +695,25 @@ inline uint64_t backtrack_8(char * qseq,
                             uint64_t offset,
                             uint64_t dirbuffersize,
                             uint64_t channel,
-                            uint64_t * alignmentlengthp)
+                            uint64_t * alignmentlengthp) -> uint64_t
 {
-  uint64_t maskup      = 1ULL << (channel+ 0);
-  uint64_t maskleft    = 1ULL << (channel+16);
-  uint64_t maskextup   = 1ULL << (channel+32);
-  uint64_t maskextleft = 1ULL << (channel+48);
+  constexpr unsigned int offset0 {0};
+  constexpr unsigned int offset1 {offset0 + 16};
+  constexpr unsigned int offset2 {offset1 + 16};
+  constexpr unsigned int offset3 {offset2 + 16};
+
+  uint64_t maskup      = 1ULL << (channel + offset0);
+  uint64_t maskleft    = 1ULL << (channel + offset1);
+  uint64_t maskextup   = 1ULL << (channel + offset2);
+  uint64_t maskextleft = 1ULL << (channel + offset3);
 
 #if 0
 
   printf("Dumping backtracking array\n");
 
-  for(uint64_t i=0; i<qlen; i++)
+  for(auto i = 0ULL; i < qlen; i++)
     {
-      for(uint64_t j=0; j<dlen; j++)
+      for(auto j = 0ULL; j < dlen; j++)
         {
           uint64_t d = dirbuffer[(offset + longestdbsequence * 4 * (j / 4)
                                   + 4 * i + (j & 3)) % dirbuffersize];
@@ -699,9 +735,9 @@ inline uint64_t backtrack_8(char * qseq,
 
   printf("Dumping gap extension array\n");
 
-  for(uint64_t i=0; i<qlen; i++)
+  for(auto i = 0ULL; i < qlen; i++)
     {
-      for(uint64_t j=0; j<dlen; j++)
+      for(auto j = 0ULL; j < dlen; j++)
         {
           uint64_t d = dirbuffer[(offset + longestdbsequence * 4 * (j / 4)
                                   + 4 * i + (j & 3)) % dirbuffersize];
@@ -728,9 +764,9 @@ inline uint64_t backtrack_8(char * qseq,
 
   int64_t i = static_cast<int64_t>(qlen) - 1;
   int64_t j = static_cast<int64_t>(dlen) - 1;
-  uint64_t aligned = 0;
-  uint64_t matches = 0;
-  char op = 0;
+  uint64_t aligned {0};
+  uint64_t matches {0};
+  char op {0};
 
 #undef SHOWALIGNMENT
 #ifdef SHOWALIGNMENT
@@ -747,20 +783,20 @@ inline uint64_t backtrack_8(char * qseq,
                      + static_cast<uint64_t>(4 * i + (j & 3)))
                     % dirbuffersize];
 
-      if ((op == 'I') && (!(d & maskextleft)))
+      if ((op == 'I') && ((d & maskextleft) == 0U))
         {
           j--;
         }
-      else if ((op == 'D') && (!(d & maskextup)))
+      else if ((op == 'D') && ((d & maskextup) == 0U))
         {
           i--;
         }
-      else if (d & maskleft)
+      else if ((d & maskleft) != 0U)
         {
           j--;
           op = 'I';
         }
-      else if (!(d & maskup))
+      else if ((d & maskup) == 0U)
         {
           i--;
           op = 'D';
@@ -768,8 +804,9 @@ inline uint64_t backtrack_8(char * qseq,
       else
         {
           if (nt_extract(qseq, static_cast<uint64_t>(i)) ==
-              nt_extract(dseq, static_cast<uint64_t>(j)))
+              nt_extract(dseq, static_cast<uint64_t>(j))) {
             matches++;
+          }
           i--;
           j--;
           op = 'M';
@@ -821,15 +858,23 @@ void search8(BYTE * * q_start,
              uint64_t dirbuffersize,
              uint64_t * dirbuffer)
 {
-  VECTORTYPE Q, R, T, M, T0, MQ, MR, MQ0;
-  VECTORTYPE *hep, **qp;
+  VECTORTYPE Q;
+  VECTORTYPE R;
+  VECTORTYPE T;
+  VECTORTYPE M;
+  VECTORTYPE T0;
+  VECTORTYPE MQ;
+  VECTORTYPE MR;
+  VECTORTYPE MQ0;
+  VECTORTYPE *hep {nullptr};
+  VECTORTYPE **qp {nullptr};
 
-  uint64_t d_pos[CHANNELS];
-  uint64_t d_offset[CHANNELS];
-  char * d_address[CHANNELS];
-  uint64_t d_length[CHANNELS];
+  uint64_t d_pos[channels];
+  uint64_t d_offset[channels];
+  char * d_address[channels];
+  uint64_t d_length[channels];
 
-  VECTORTYPE dseqalloc[CDEPTH];
+  VECTORTYPE dseqalloc[cdepth];
 
   VECTORTYPE H0;
   VECTORTYPE F0;
@@ -837,9 +882,9 @@ void search8(BYTE * * q_start,
 
   BYTE * dseq = reinterpret_cast<BYTE*>(& dseqalloc);
 
-  int64_t seq_id[CHANNELS];
-  uint64_t next_id = 0;
-  uint64_t done;
+  int64_t seq_id[channels];
+  uint64_t next_id {0};
+  uint64_t done {0};
 
 #ifdef __aarch64__
   const VECTORTYPE T0_init = { -1, 0, 0, 0, 0, 0, 0, 0,
@@ -862,7 +907,7 @@ void search8(BYTE * * q_start,
   hep = CAST_VECTOR_p(hearray);
   qp = reinterpret_cast<VECTORTYPE**>(q_start);
 
-  for (int c=0; c<CHANNELS; c++)
+  for(auto c = 0U; c < channels; c++)
     {
       d_address[c] = nullptr;
       d_pos[c] = 0;
@@ -873,36 +918,43 @@ void search8(BYTE * * q_start,
   F0 = v_zero;
   H0 = v_zero;
 
-  int easy = 0;
+  bool easy {false};
 
   uint64_t * dir = dirbuffer;
 
-  while(1)
+  while(true)
     {
       if (easy)
         {
           // fill all channels
 
-          for(int c=0; c<CHANNELS; c++)
+          for(auto c = 0U; c < channels; c++)
             {
-              for(int j=0; j<CDEPTH; j++)
+              for(auto j = 0U; j < cdepth; j++)
                 {
-                  if (d_pos[c] < d_length[c])
-                    dseq[CHANNELS*j+c]
+                  if (d_pos[c] < d_length[c]) {
+                    dseq[channels * j + c]
                       = 1 + nt_extract(d_address[c], d_pos[c]++);
-                  else
-                    dseq[CHANNELS*j+c] = 0;
+                  }
+                  else {
+                    dseq[channels * j + c] = 0;
+                  }
                 }
-              if (d_pos[c] == d_length[c])
-                easy = 0;
+              if (d_pos[c] == d_length[c]) {
+                easy = false;
+              }
             }
 
 #ifdef __x86_64__
-          if (ssse3_present)
-            dprofile_shuffle8(dprofile, score_matrix, dseq);
+          if (ssse3_present != 0)
+            {
+              dprofile_shuffle8(dprofile, score_matrix, dseq);
+            }
           else
 #endif
-            dprofile_fill8(dprofile, score_matrix, dseq);
+            {
+              dprofile_fill8(dprofile, score_matrix, dseq);
+            }
 
           align_cells_regular_8(S, hep, qp, &Q, &R, qlen, &F0, dir, &H0);
         }
@@ -911,26 +963,29 @@ void search8(BYTE * * q_start,
           // One or more sequences ended in the previous block
           // We have to switch over to a new sequence
 
-          easy = 1;
+          easy = true;
 
           M = v_zero;
           T = T0;
-          for (unsigned int c = 0; c < CHANNELS; c++)
+          for(auto c = 0U; c < channels; c++)
             {
               if (d_pos[c] < d_length[c])
                 {
                   // this channel has more sequence
 
-                  for(unsigned int j = 0; j < CDEPTH; j++)
+                  for(auto j = 0U; j < cdepth; j++)
                     {
-                      if (d_pos[c] < d_length[c])
-                        dseq[CHANNELS * j + c]
+                      if (d_pos[c] < d_length[c]) {
+                        dseq[channels * j + c]
                           = 1 + nt_extract(d_address[c], d_pos[c]++);
-                      else
-                        dseq[CHANNELS*j+c] = 0;
+                      }
+                      else {
+                        dseq[channels * j + c] = 0;
+                      }
                     }
-                  if (d_pos[c] == d_length[c])
-                    easy = 0;
+                  if (d_pos[c] == d_length[c]) {
+                    easy = false;
+                  }
                 }
               else
                 {
@@ -947,14 +1002,14 @@ void search8(BYTE * * q_start,
 
                       char * dbseq = reinterpret_cast<char*>(d_address[c]);
                       uint64_t dbseqlen = d_length[c];
-                      uint64_t z = (dbseqlen+3) % 4;
+                      uint64_t z = (dbseqlen + 3) % 4;
                       uint64_t score
-                        = (reinterpret_cast<BYTE*>(S))[z * CHANNELS + c];
+                        = (reinterpret_cast<BYTE*>(S))[z * channels + c];
                       scores[cand_id] = score;
 
-                      uint64_t diff;
+                      uint64_t diff {0};
 
-                      if (score < 255)
+                      if (score < UINT8_MAX)
                         {
                           uint64_t offset = d_offset[c];
                           diff = backtrack_8(query.seq, dbseq, qlen, dbseqlen,
@@ -965,7 +1020,7 @@ void search8(BYTE * * q_start,
                         }
                       else
                         {
-                          diff = 255;
+                          diff = UINT8_MAX;
                         }
 
                       diffs[cand_id] = diff;
@@ -978,8 +1033,8 @@ void search8(BYTE * * q_start,
                       // get next sequence
                       seq_id[c] = static_cast<int64_t>(next_id);
                       uint64_t seqno = seqnos[next_id];
-                      char* address;
-                      unsigned int length;
+                      char * address {nullptr};
+                      unsigned int length {0};
 
                       db_getsequenceandlength(seqno, & address, & length);
 
@@ -994,15 +1049,18 @@ void search8(BYTE * * q_start,
                       (reinterpret_cast<BYTE*>(&F0))[c] = 2 * gap_open_penalty + 2 * gap_extend_penalty;
 
                       // fill channel
-                      for(unsigned int j = 0; j < CDEPTH; j++)
+                      for(auto j = 0U; j < cdepth; j++)
                         {
-                          if (d_pos[c] < d_length[c])
-                            dseq[CHANNELS*j+c] = 1 + nt_extract(d_address[c], d_pos[c]++);
-                          else
-                            dseq[CHANNELS*j+c] = 0;
+                          if (d_pos[c] < d_length[c]) {
+                            dseq[channels * j + c] = 1 + nt_extract(d_address[c], d_pos[c]++);
+                          }
+                          else {
+                            dseq[channels * j + c] = 0;
+                          }
                         }
-                      if (d_pos[c] == d_length[c])
-                        easy = 0;
+                      if (d_pos[c] == d_length[c]) {
+                        easy = false;
+                      }
                     }
                   else
                     {
@@ -1011,23 +1069,29 @@ void search8(BYTE * * q_start,
                       d_address[c] = nullptr;
                       d_pos[c] = 0;
                       d_length[c] = 0;
-                      for (unsigned int j=0; j<CDEPTH; j++)
-                        dseq[CHANNELS*j+c] = 0;
+                      for(auto j = 0U; j < cdepth; j++) {
+                        dseq[channels * j + c] = 0;
+                      }
                     }
                 }
 
               T = v_shift_left(T);
             }
 
-          if (done == sequences)
+          if (done == sequences) {
             break;
+          }
 
 #ifdef __x86_64__
-          if (ssse3_present)
-            dprofile_shuffle8(dprofile, score_matrix, dseq);
+          if (ssse3_present != 0)
+            {
+              dprofile_shuffle8(dprofile, score_matrix, dseq);
+            }
           else
 #endif
-            dprofile_fill8(dprofile, score_matrix, dseq);
+            {
+              dprofile_fill8(dprofile, score_matrix, dseq);
+            }
 
           MQ = v_and(M, Q);
           MR = v_and(M, R);
@@ -1042,8 +1106,9 @@ void search8(BYTE * * q_start,
       H0 = v_sub(F0, Q);
       F0 = v_add(F0, R);
 
-      dir += 4*longestdbsequence;
-      if (dir >= dirbuffer + dirbuffersize)
+      dir += 4 * longestdbsequence;
+      if (dir >= dirbuffer + dirbuffersize) {
         dir -= dirbuffersize;
+      }
     }
 }
