@@ -1,7 +1,7 @@
 /*
     SWARM
 
-    Copyright (C) 2012-2019 Torbjorn Rognes and Frederic Mahe
+    Copyright (C) 2012-2021 Torbjorn Rognes and Frederic Mahe
 
     This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU Affero General Public License as
@@ -39,9 +39,9 @@ private:
     int64_t work; /* 1: work available, 0: wait, -1: quit */
   } * thread_array;
 
-  static void * worker(void * vp)
+  static auto worker(void * vp) -> void *
   {
-    struct thread_s * tip = static_cast<struct thread_s *>(vp);
+    auto * tip = static_cast<struct thread_s *>(vp);
 
     pthread_mutex_lock(&tip->workmutex);
 
@@ -49,8 +49,9 @@ private:
     while (tip->work >= 0)
       {
         /* wait for work available */
-        if (tip->work == 0)
+        if (tip->work == 0) {
           pthread_cond_wait(&tip->workcond, &tip->workmutex);
+        }
 
         if (tip->work > 0)
           {
@@ -78,7 +79,7 @@ public:
       (xmalloc(static_cast<uint64_t>(thread_count) * sizeof(struct thread_s)));
 
     /* init and create worker threads */
-    for(int64_t i=0; i<thread_count; i++)
+    for(auto i = 0LL; i < thread_count; i++)
       {
         struct thread_s * tip = thread_array + i;
         tip->t = i;
@@ -89,8 +90,9 @@ public:
         if (pthread_create(&tip->pthread,
                            &attr,
                            worker,
-                           static_cast<void*>(thread_array + i)))
+                           static_cast<void*>(thread_array + i)) != 0) {
           fatal("Cannot create thread");
+        }
       }
   }
 
@@ -101,7 +103,7 @@ public:
     /* destroy threads */
     /* finish and clean up worker threads */
 
-    for(int64_t i=0; i<thread_count; i++)
+    for(auto i = 0LL; i < thread_count; i++)
       {
         struct thread_s * tip = thread_array + i;
 
@@ -112,8 +114,9 @@ public:
         pthread_mutex_unlock(&tip->workmutex);
 
         /* wait for worker to quit */
-        if (pthread_join(tip->pthread, nullptr))
+        if (pthread_join(tip->pthread, nullptr) != 0) {
           fatal("Cannot join thread");
+        }
 
         pthread_cond_destroy(&tip->workcond);
         pthread_mutex_destroy(&tip->workmutex);
@@ -126,7 +129,7 @@ public:
   void run()
   {
     /* wake up threads */
-    for(int64_t i=0; i<thread_count; i++)
+    for(auto i = 0LL; i < thread_count; i++)
       {
         struct thread_s * tip = thread_array + i;
         pthread_mutex_lock(&tip->workmutex);
@@ -136,12 +139,13 @@ public:
       }
 
     /* wait for threads to finish their work */
-    for(int64_t i=0; i<thread_count; i++)
+    for(auto i = 0LL; i < thread_count; i++)
       {
         struct thread_s * tip = thread_array + i;
         pthread_mutex_lock(&tip->workmutex);
-        while (tip->work > 0)
+        while (tip->work > 0) {
           pthread_cond_wait(&tip->workcond, &tip->workmutex);
+        }
         pthread_mutex_unlock(&tip->workmutex);
       }
   }
